import os
import sys
import shutil
from subprocess import call, DEVNULL
from annogesiclib.multiparser import Multiparser
from annogesiclib.converter import Converter
from annogesiclib.format_fixer import FormatFixer
from annogesiclib.helper import Helper


class RATT(object):
    '''annotation transfer'''

    def __init__(self, args_ratt):
        self.multiparser = Multiparser()
        self.converter = Converter()
        self.format_fixer = FormatFixer()
        self.helper = Helper()
        if args_ratt.ref_gbk:
            self.gbk = os.path.join(args_ratt.ref_gbk, "gbk_tmp")
            self.gbk_tmp = os.path.join(self.gbk, "tmp")
            self.embl = os.path.join(args_ratt.ref_gbk, "embls")
        if args_ratt.ref_embls:
            self.embl = args_ratt.ref_embls
        self.ratt_log = os.path.join(args_ratt.output_path, "ratt_log.txt")
        self.tmp_files = {"tar": os.path.join(args_ratt.tar_fastas, "tmp"),
                          "ref": os.path.join(args_ratt.ref_fastas, "tmp"),
                          "out_gff": os.path.join(args_ratt.gff_outfolder,
                                                  "tmp"),
                          "gff": os.path.join(args_ratt.gff_outfolder,
                                              "tmp.gff"),
                          "ptt": os.path.join(args_ratt.gff_outfolder,
                                              "tmp.ptt"),
                          "rnt": os.path.join(args_ratt.gff_outfolder,
                                              "tmp.rnt")}

    def _convert_to_pttrnt(self, gffs, files):
        for gff in files:
            if gff.endswith(".gff"):
                gff = os.path.join(gffs, gff)
                filename = gff.split("/")
                prefix = filename[-1][:-4]
                rnt = gff[:-3] + "rnt"
                ptt = gff[:-3] + "ptt"
                fasta = self.helper.get_correct_file(self.tmp_files["tar"],
                                                     ".fa", prefix, None, None)
                if fasta:
                    self.converter.convert_gff2rntptt(gff, fasta, ptt, rnt,
                                                      None, None)

    def _remove_files(self, args_ratt, out_gbk):
        self.helper.remove_all_content(args_ratt.gff_outfolder, ".gff", "file")
        self.helper.remove_all_content(args_ratt.gff_outfolder, ".ptt", "file")
        self.helper.remove_all_content(args_ratt.gff_outfolder, ".rnt", "file")
        self.helper.move_all_content(self.tmp_files["out_gff"],
                                     args_ratt.gff_outfolder, None)
        shutil.rmtree(self.tmp_files["out_gff"])
        shutil.rmtree(self.tmp_files["tar"])
        shutil.rmtree(self.tmp_files["ref"])
        self.helper.remove_tmp_dir(args_ratt.tar_fastas)
        self.helper.remove_tmp_dir(args_ratt.ref_fastas)
        self.helper.remove_tmp_dir(args_ratt.ref_embls)
        self.helper.remove_tmp_dir(args_ratt.ref_gbk)

    def _convert_to_gff(self, ratt_result, args_ratt, files):
        name = ratt_result.split(".")
        filename = ".".join(name[1:-2]) + ".gff"
        output_file = os.path.join(args_ratt.output_path, filename)
        self.converter.convert_embl2gff(
             os.path.join(args_ratt.output_path, ratt_result), output_file)
        self.format_fixer.fix_ratt(output_file, ".".join(name[1:-2]),
                                   "tmp_gff")
        shutil.move("tmp_gff", output_file)
        shutil.copy(output_file, os.path.join(args_ratt.gff_outfolder,
                                              filename))
        files.append(filename)

    def _parser_embl_gbk(self, files):
        self.helper.check_make_folder(self.gbk)
        for file_ in files:
            close = False
            with open(file_, "r") as f_h:
                for line in f_h:
                    if (line.startswith("LOCUS")):
                        out = open(self.gbk_tmp, "w")
                        datas = line.split(" ")
                        for data in datas:
                            if (len(data) != 0) and (data != "LOCUS"):
                                filename = ".".join([data, "gbk"])
                                break
                    elif (line.startswith("VERSION")):
                        datas = line.split(" ")
                        for data in datas:
                            if (len(data) != 0) and (data != "VERSION"):
                                new_filename = ".".join([data, "gbk"])
                                break
                        if new_filename.find(filename):
                            filename = new_filename
                    if out:
                        out.write(line)
                    if line.startswith("//"):
                        out.close()
                        close = True
                        shutil.move(self.gbk_tmp,
                                    os.path.join(self.gbk, filename))
            if not close:
                out.close()
        return self.gbk

    def _convert_embl(self, ref_embls):
        '''convert gbk to embl'''
        detect_gbk = False
        gbks = []
        out_gbk = None
        for embl in os.listdir(ref_embls):
            if (embl.endswith(".gbk")) or (
                    embl.endswith(".gbff")) or (
                    embl.endswith(".gb")):
                detect_gbk = True
                gbks.append(os.path.join(ref_embls, embl))
        if not detect_gbk:
            print("Error: Please assign proper Genebank files!")
            sys.exit()
        elif detect_gbk:
            out_gbk = self._parser_embl_gbk(gbks)
            self.converter.convert_gbk2embl(out_gbk)
            self.helper.check_make_folder(self.embl)
            self.helper.move_all_content(out_gbk, self.embl, [".embl"])
        return out_gbk

    def _run_ratt(self, args_ratt, tar, ref, out):
        call([args_ratt.ratt_path, self.embl,
              os.path.join(self.tmp_files["tar"], tar + ".fa"),
              args_ratt.element, args_ratt.transfer_type,
              os.path.join(self.tmp_files["ref"], ref + ".fa")],
             stdout=out, stderr=DEVNULL)

    def _format_and_run(self, args_ratt):
        print("Running RATT")
        for pair in args_ratt.pairs:
            ref = pair.split(":")[0]
            tar = pair.split(":")[1]
            out = open(self.ratt_log, "w+")
            self._run_ratt(args_ratt, tar, ref, out)
            for filename in os.listdir():
                if ("final" in filename):
                    shutil.move(filename, os.path.join(args_ratt.output_path,
                                                       filename))
                elif (args_ratt.element in filename) or (
                      "query" in filename) or (
                      "Reference" in filename) or (
                      "Query" in filename) or (
                      "Sequences" in filename):
                    if os.path.isfile(filename):
                        os.remove(filename)
                    if os.path.isdir(filename):
                        shutil.rmtree(filename)
        out.close()

    def annotation_transfer(self, args_ratt):
        self.multiparser.parser_fasta(args_ratt.tar_fastas)
        self.multiparser.parser_fasta(args_ratt.ref_fastas)
        out_gbk = None
        if args_ratt.ref_embls is None:
            out_gbk = self._convert_embl(args_ratt.ref_gbk)
        self._format_and_run(args_ratt)
        if args_ratt.convert:
            files = []
            for data in os.listdir(args_ratt.output_path):
                if "final.embl" in data:
                    self._convert_to_gff(data, args_ratt, files)
                    self._convert_to_pttrnt(args_ratt.gff_outfolder, files)
            self.helper.check_make_folder(self.tmp_files["out_gff"])
            for folder in os.listdir(args_ratt.tar_fastas):
                files = []
                if "_folder" in folder:
                    datas = folder.split("_folder")
                    prefix = datas[0][:-3]
                    for file_ in os.listdir(os.path.join(args_ratt.tar_fastas,
                                                         folder)):
                        files.append(file_[:-3])
                    for gff in os.listdir(args_ratt.gff_outfolder):
                        for file_ in files:
                            if (".gff" in gff) and (file_ == gff[:-4]):
                                self.helper.merge_file(os.path.join(
                                     args_ratt.gff_outfolder, gff),
                                     self.tmp_files["gff"])
                            if (".ptt" in gff) and (file_ == gff[:-4]):
                                self.helper.merge_file(os.path.join(
                                     args_ratt.gff_outfolder, gff),
                                     self.tmp_files["ptt"])
                            if (".rnt" in gff) and (file_ == gff[:-4]):
                                self.helper.merge_file(os.path.join(
                                     args_ratt.gff_outfolder, gff),
                                     self.tmp_files["rnt"])
                    if os.path.exists(self.tmp_files["gff"]):
                        shutil.move(self.tmp_files["gff"], os.path.join(
                                    self.tmp_files["out_gff"], prefix + ".gff"))
                        shutil.move(self.tmp_files["ptt"], os.path.join(
                                    self.tmp_files["out_gff"], prefix + ".ptt"))
                        shutil.move(self.tmp_files["rnt"], os.path.join(
                                    self.tmp_files["out_gff"], prefix + ".rnt"))
                    else:
                        print("Error: Please check your output fasta or "
                              "annotation files, they should only contain "
                              "the query genome. And make sure your RATT can "
                              "work properly (check $ANNOgesic/output/"
                              "annotation_transfer/ratt_log.txt).")
        self._remove_files(args_ratt, out_gbk)



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def filter_utr(srna_gff, srna_table, min_utr):
    out = open("tmp_utr_srna.gff", "w")
    out_ta = open("tmp_utr_srna.csv", "w")
    out.write("##gff-version 3\n")
    gffs = []
    tables = []
    gff_parser = Gff3Parser()
    g_f = open(srna_gff, "r")
    for entry in gff_parser.entries(g_f):
        gffs.append(entry)
    fh = open(srna_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        if row[0] != "rank":
            if (float(row[7]) >= min_utr):
                tables.append(row)
                out_ta.write("\t".join(row) + "\n")
    for gff in gffs:
        for table in tables:
            if (table[0] == gff.seq_id) and (
                    int(table[2]) == gff.start) and (
                    int(table[3]) == gff.end) and (
                    table[4] == gff.strand):
                out.write(gff.info + "\n")
    g_f.close()
    fh.close()
    os.remove(srna_gff)
    os.remove(srna_table)
    shutil.move("tmp_utr_srna.gff", srna_gff)
    shutil.move("tmp_utr_srna.csv", srna_table)
    out.close()
    out_ta.close()



import os
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper
from annogesiclib.parser_wig import WigParser
from annogesiclib.gen_TSS_type import compare_tss_cds, fix_primary_type
from annogesiclib.lib_reader import read_libs, read_wig


def get_upstream(seq, tss, out, name, nt_before):
    if tss.strand == "+":
        if (tss.start - nt_before + 1) <= 0:
            start = 1
        else:
            start = tss.start - nt_before + 1
        fasta = Helper().extract_gene(seq, start,
                                      tss.start, tss.strand)
    else:
        if (tss.start + nt_before - 1) > len(seq):
            end = len(seq)
        else:
            end = tss.start + nt_before - 1
        fasta = Helper().extract_gene(seq, tss.start,
                                      end, tss.strand)
    out.write("{0}\n{1}\n".format(name, fasta))


def print_fasta(seq, tss, files, name, nt_before):
    for key in seq.keys():
        if tss.seq_id == key:
            if "Primary" in tss.attributes["type"]:
                get_upstream(seq[key], tss, files["pri"], name, nt_before)
            if "Secondary" in tss.attributes["type"]:
                get_upstream(seq[key], tss, files["sec"], name, nt_before)
            if "Internal" in tss.attributes["type"]:
                get_upstream(seq[key], tss, files["inter"], name, nt_before)
            if "Antisense" in tss.attributes["type"]:
                get_upstream(seq[key], tss, files["anti"], name, nt_before)
            if "Orphan" in tss.attributes["type"]:
                get_upstream(seq[key], tss, files["orph"], name, nt_before)


def read_data(tss_file, fasta_file):
    seq = {}
    tsss = []
    t_f = open(tss_file, "r")
    for entry in Gff3Parser().entries(t_f):
        tsss.append(entry)
    with open(fasta_file, "r") as f_h:
        for line in f_h:
            line = line.strip()
            if line[0] == ">":
                seq[line[1:]] = ""
                seq_id = line[1:]
            else:
                seq[seq_id] = seq[seq_id] + line
    tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return tsss, seq


def read_gff(gff_file):
    cdss = []
    genes = []
    g_f = open(gff_file, "r")
    for entry in Gff3Parser().entries(g_f):
        if (entry.feature == "CDS") or \
           (entry.feature == "rRNA") or \
           (entry.feature == "tRNA"):
            cdss.append(entry)
        if entry.feature == "gene":
            genes.append(entry)
    cdss = sorted(cdss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return cdss, genes


def upstream(tss_file, fasta_file, gff_file, out_class, args_pro, prefix):
    '''get the upstream sequence of TSS'''
    files = {"pri": open("tmp/primary.fa", "w"),
             "sec": open("tmp/secondary.fa", "w"),
             "inter": open("tmp/internal.fa", "w"),
             "anti": open("tmp/antisense.fa", "w"),
             "orph": open("tmp/orphan.fa", "w")}
    tsss, seq = read_data(tss_file, fasta_file)
    num_tss = 0
    if not args_pro.source:
        out = open(out_class, "w")
        out.write("##gff-version 3\n")
        cdss, genes = read_gff(gff_file)
    for tss in tsss:
        if args_pro.source is True:
            name = ">" + "_".join([str(tss.start), tss.strand, tss.seq_id])
            print_fasta(seq, tss, files, name, args_pro.nt_before)
        else:
            tss_type = compare_tss_cds(tss, cdss, genes)
            tss.attributes = tss_type[1]
            tss.attributes["ID"] = tss.seq_id + "_tss" + str(num_tss)
            tss.attribute_string = "".join([
                tss_type[0], ";ID=", tss.seq_id, "_tss", str(num_tss)])
            num_tss += 1
    if not args_pro.source:
        libs, texs = read_libs(args_pro.input_libs, args_pro.tex_wigs)
        wigs_f = read_wig(os.path.join(
            args_pro.wig_path, prefix + "_forward.wig"), "+", libs)
        wigs_r = read_wig(os.path.join(
            args_pro.wig_path, prefix + "_reverse.wig"), "+", libs)
        sort_tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start,
                                                k.end, k.strand))
        final_tsss = fix_primary_type(sort_tsss, wigs_f, wigs_r)
        for tss in final_tsss:
            name = ">" + "_".join([str(tss.start), tss.strand, tss.seq_id])
            tss.attribute_string = ";".join(
                ["=".join(items) for items in tss.attributes.items()])
            out.write("\t".join([str(field) for field in [
                            tss.seq_id, tss.source, tss.feature, tss.start,
                            tss.end, tss.score, tss.strand, tss.phase,
                            tss.attribute_string]]) + "\n")
            print_fasta(seq, tss, files, name, args_pro.nt_before)


def del_repeat_fasta(input_file, out_file):
    data = {}
    check_same = False
    first_file = True
    out = open(out_file, "w")
    with open(input_file, "r") as f_h:
        for line in f_h:
            line = line.strip()
            if line[0] == ">":
                if check_same:
                    check_same = False
                if first_file:
                    seq_id = line[1:]
                    first_file = False
                    data[seq_id] = ""
                else:
                    if line[1:] in data.keys():
                        check_same = True
                    else:
                        seq_id = line[1:]
                        data[seq_id] = ""
            else:
                if check_same:
                    pass
                else:
                    data[seq_id] = data[seq_id] + line
    for strain, fasta in data.items():
        out.write(">" + strain + "\n")
        out.write(fasta + "\n")



import os
import networkx as nx
import matplotlib
matplotlib.use('Agg')
from matplotlib.offsetbox import AnchoredText
import matplotlib.pyplot as plt
plt.rcParams['image.cmap'] = 'RdBu_r'


def node(item, nodes, center, colors, labels1, labels2):
    if item not in nodes:
        nodes.append(item)
        if len(item) > 5:
            labels2[item] = item
            labels1[item] = ""
        else:
            labels1[item] = item
            labels2[item] = ""
        if (center["locus_tag"] == item) or \
           (center["gene_name"] == item):
            colors[item] = '#FFFF66'
        else:
            colors[item] = '#CCFFCC'


def get_largest_compare(tick, score):
    same = True
    if (score >= 20) and tick >= 20:
        pass
    else:
        if score != tick:
            same = False
    if score > tick:
        if score >= 20:
            tick = 20
        else:
            tick = score
    return (tick, same)


def add_edge(G, ppi, style, weight, colorppi):
    G.add_edge(ppi["item_a"], ppi["item_b"],
               color=float(colorppi), style=style, weight=weight)


def add_node(G, nodes):
    G.add_nodes_from(nodes)


def best_assign_attributes(check_na, G, ppi, pre_ppi, first, style):
    check_na["best"] = True
    if ppi["score"] == 0:
        if ppi["below"] >= 20:
            weight = 22
        else:
            weight = ppi["below"] + 1
    else:
        if ppi["score"] >= 20:
            weight = 22
        else:
            weight = ppi["score"] + ppi["below"] + 1
    add_edge(G, ppi, style, weight, ppi["best"])
    if not first:
        if pre_ppi["best"] != ppi["best"]:
            check_na["same_best"] = True


def create_node(ppis, scores, nodes, center, colors, labels1, labels2, edges,
                G, cutoff_score, check_na, pre_ppi):
    first = True
    for ppi in ppis:
        scores.append(ppi["score"])
        node(ppi["item_a"], nodes, center, colors, labels1, labels2)
        node(ppi["item_b"], nodes, center, colors, labels1, labels2)
        if ((ppi["item_a"], ppi["item_b"]) not in edges) or \
           ((ppi["item_b"], ppi["item_a"]) not in edges):
            edges.append((ppi["item_a"], ppi["item_b"]))
            if ppi["best"] == "NA":
                add_edge(G, ppi, 'dashed', 1, -1)
            elif float(ppi["best"]) <= cutoff_score:
                best_assign_attributes(check_na, G, ppi,
                                       pre_ppi, first, "dashdot")
            else:
                best_assign_attributes(check_na, G, ppi,
                                       pre_ppi, first, "solid")
            pre_ppi = ppi
            first = False
    add_node(G, nodes)
    return pre_ppi


def modify_label(labels2, new_labels):
    for key, value in labels2.items():
        if "_" in value:
            new_labels[key] = value.replace("_", "\n")
        else:
            new_labels[key] = value


def plot_text(check_na, plt, ppis, ppi, color_edge):
    na = False
    if check_na["na"]:
        na = True
    elif check_na["best"]:
        if len(ppis) < 2:
            na = True
        else:
            cbar = plt.colorbar(color_edge)
            cbar.ax.tick_params(labelsize=16)
    return na


def nx_node(G, pos, node_size, colors, color_list):
    '''draw the node'''
    nx.draw_networkx_nodes(G, pos, node_size=node_size, node_shape='o',
                           nodelist=colors.keys(), node_color=color_list,
                           linewidths=1)


def nx_edge(G, pos, edges, colors, styles, weights):
    '''draw the edge'''
    color_edge = (nx.draw_networkx_edges(G, pos, edges=edges,
                  edge_color=colors, style=styles, width=weights,
                  edge_vmin=-1, edge_vmax=1, vmin=-1, vmax=1))
    return color_edge


def nx_label(G, pos, labels, size):
    '''setup the label of network'''
    nx.draw_networkx_labels(G, pos, labels, font_size=size, font_weight='bold')


def nx_color_style(G, edges):
    '''setup the color of network'''
    colors = []
    styles = []
    check_na = True
    for u, v in edges:
        colors.append(G[u][v]['color'])
        styles.append(G[u][v]['style'])
        if (G[u][v]['style'] == "solid") or (
                G[u][v]['style'] == "dashdot"):
            check_na = False
    return colors, styles, check_na


def print_title(plt, na, center):
    if not na:
        plt.title("|".join([center["locus_tag"],
                           " ".join([center["gene_name"],
                                     "(based on the score of best literature)"])]),
                  fontsize="16")
    else:
        plt.title("|".join([center["locus_tag"],
                           " ".join([center["gene_name"],
                                     "(based on the score of best literature)"])]) + \
                  "\n the numbers of supported literatures in all interactions are 0",
                  fontsize="16")

def plot(ppis, center, strain, cutoff_score, node_size, out_folder):
    nodes = []
    edges = []
    labels1 = {}
    labels2 = {}
    colors = {}
    check_na = {"number": False, "best": False, "na": False,
                "same_number": False, "same_best": False}
    pre_ppi = ""
    scores = []
    weights = []
    plt.figure(figsize=(15, 15))
    G = nx.Graph()
    pre_ppi = create_node(ppis, scores, nodes, center, colors,
                          labels1, labels2, edges, G,
                          cutoff_score, check_na, pre_ppi)
    pos = nx.spring_layout(G, k=2, scale=3, iterations=20)
    color_list = []
    for color in colors.values():
        color_list.append(color)
    nx_node(G, pos, node_size, colors, color_list)
    connects = G.edges()
    for weight in G.edges(data=True):
        if weight[2]["weight"] <= 30:
            weights.append(weight[2]["weight"])
        else:
            weights.append(30)
    colors, styles, check_na["na"] = nx_color_style(G, connects)
    color_edge = nx_edge(G, pos, connects, colors, styles, weights)
    nx_label(G, pos, labels1, 12)
    new_labels = {}
    modify_label(labels2, new_labels)
    nx_label(G, pos, new_labels, 10)
    na = plot_text(check_na, plt, ppis, pre_ppi, color_edge)
    print_title(plt, na, center)
    plt.axis('off')
    if strain not in os.listdir(out_folder):
        os.mkdir(os.path.join(out_folder, strain))
    plt.savefig(os.path.join(out_folder, strain,
                "_".join([center["locus_tag"], center["gene_name"] + ".png"])),
                bbox_inches="tight")
    plt.clf()
    plt.close('all')
    return check_na

def score_compare(score, scores, cutoff_score, ppi):
    '''check the number of literatures which are pass the cutoff'''
    if score == "NA":
        ppi["score"] = 0
        ppi["below"] = 0
    elif float(score) >= cutoff_score:
        scores["score"] += 1
    else:
        scores["below"] += 1


def assign_score_below(pre_ppi, scores, ppis):
    if "score" not in pre_ppi.keys():
        pre_ppi["score"] = scores["score"]
    if "below" not in pre_ppi.keys():
        pre_ppi["below"] = scores["below"]
    ppis.append(pre_ppi)


def get_best(pre_ppi, ppi, row):
    '''get the best score of PPI'''
    if "best" not in pre_ppi.keys():
        ppi["best"] = row[8]
    else:
        if pre_ppi["best"] == "NA":
            ppi["best"] = row[8]
        else:
            if float(pre_ppi["best"]) < float(row[8]):
                ppi["best"] = row[8]
            else:
                ppi["best"] = pre_ppi["best"]


def interaction(first, pre_ppi, scores, ppis, match, center, cutoff_score,
                node_size, out_folder):
    '''check the interaction of two proteins'''
    if first:
        pass
    else:
        assign_score_below(pre_ppi, scores, ppis)
        if match:
            plot(ppis, center, pre_ppi["strain"], cutoff_score,
                 node_size, out_folder)
            match = False
        else:
            print("No interacted partner with {0} | {1}".format(
                  center["locus_tag"], center["gene_name"]))
        scores = {"score": 0, "below": 0}
        ppis = []
        first = True
    return first, scores, match, ppis


def plot_ppi(PPI_file, cutoff_score, out_folder, node_size):
    '''plot the network of PPI'''
    ppis = []
    first = True
    pre_ppi = None
    scores = {"score": 0, "below": 0}
    center = {}
    start = False
    match = False
    with open(PPI_file) as fh:
        for line in fh:
            line = line.strip()
            row = line.split("\t")
            start = True
            if row[0].startswith("Interaction"):
                first, scores, match, ppis = interaction(
                        first, pre_ppi, scores, ppis, match, center,
                        cutoff_score, node_size, out_folder)
                datas = row[0].split(" | ")
                center["locus_tag"] = datas[0].split(" ")[-1]
                center["gene_name"] = datas[-1]
                print("Plotting {0}".format(center["gene_name"]))
            elif row[0] == "Genome":
                pass
            else:
                ppi = {"strain": row[0], "item_a": row[1], "item_b": row[2],
                       "mode": row[3]}
                if (ppi["item_a"] == center["locus_tag"]) or (
                        ppi["item_a"] == center["gene_name"]) or (
                        ppi["item_b"] == center["locus_tag"]) or (
                        ppi["item_b"] == center["gene_name"]):
                    match = True
                if first:
                    first = False
                    score_compare(row[8], scores, cutoff_score, ppi)
                    ppi["best"] = row[8]
                else:
                    if (ppi["strain"] == pre_ppi["strain"]) and (
                            ppi["item_a"] == pre_ppi["item_a"]) and (
                            ppi["item_b"] == pre_ppi["item_b"]):
                        get_best(pre_ppi, ppi, row)
                        score_compare(row[8], scores, cutoff_score, ppi)
                    else:
                        assign_score_below(pre_ppi, scores, ppis)
                        scores = {"score": 0, "below": 0}
                        score_compare(row[8], scores, cutoff_score, ppi)
                        ppi["best"] = row[8]
                pre_ppi = ppi
    if start and match:
        assign_score_below(pre_ppi, scores, ppis)
        plot(ppis, center, pre_ppi["strain"],
             cutoff_score, node_size, out_folder)
    elif not start:
        print("No proper result can be retrieved in " + PPI_file)
    elif not match:
        print("No interacted partner with {0} | {1}".format(
               center["locus_tag"], center["gene_name"]))



import os
import sys
import csv
import time
from subprocess import call
from annogesiclib.helper import Helper
from annogesiclib.plot_PPI import plot_ppi
from annogesiclib.converter import Converter
from annogesiclib.gff3 import Gff3Parser


class PPINetwork(object):
    '''detection of PPI'''

    def __init__(self, out_folder):
        self.helper = Helper()
        self.converter = Converter()
        self.gffparser = Gff3Parser()
        self.tmp_id = os.path.join(out_folder, "tmp_id_list")
        self.all_result = os.path.join(out_folder, "all_results")
        self.best_result = os.path.join(out_folder, "best_results")
        self.fig = os.path.join(out_folder, "figures")
        self.ref_tags = {}
        self.with_strain = "with_strain"
        self.without_strain = "without_strain"
        self.tmp_files = {"log": "tmp_log", "action": "tmp_action.log",
                          "pubmed": "tmp_pubmed.log",
                          "specific": os.path.join(
                                      out_folder, "tmp_specific"),
                          "nospecific": os.path.join(
                                        out_folder, "tmp_nospecific"),
                          "wget_action": os.path.join(
                                         out_folder, "tmp_action")}

    def _make_folder_no_exist(self, path, folder):
        if folder not in os.listdir(path):
            os.mkdir(os.path.join(path, folder))

    def _make_subfolder(self, path, strain, ptt):
        os.mkdir(os.path.join(path, strain))
        os.mkdir(os.path.join(path, strain, ptt))

    def _run_wget(self, source, folder, log):
        call(["wget", source, "-O", folder], stderr=log)
        time.sleep(2)

    def _wget_id(self, strain, locus, strain_id, files):
        detect_id = False
        if strain == strain_id["ptt"]:
            print("Retrieving STRING ID for {0} of {1} -- {2}".format(
                   locus, strain_id["string"], strain_id["file"]))
            id_source = ("http://string-db.org/api/tsv/resolve?"
                         "identifier={0}&species={1}").format(
                         locus, strain_id["string"])
            self._run_wget(id_source, os.path.join(files["id_list"], locus),
                           files["id_log"])
            detect_id = True
        return detect_id

    def _retrieve_id(self, strain_id, genes, files):
        for gene in genes:
            if gene["gene"] != "-":
                detect_id = self._wget_id(gene["strain"], gene["gene"],
                                          strain_id, files)
                self.ref_tags[gene["gene"]] = gene["locus_tag"]
            else:
                detect_id = self._wget_id(gene["strain"], gene["locus_tag"],
                                          strain_id, files)
                self.ref_tags[gene["locus_tag"]] = gene["locus_tag"]
            if not detect_id:
                print("Error: There is no {0} in {1}".format(
                       gene, strain_id["file"]))

    def _get_prefer_name(self, row_a, strain_id, files, querys):
        prefername = ""
        filename = row_a.split(".")
        if ((filename[1] not in os.listdir(files["id_list"])) and (
                "all" not in querys)) or ("all" in querys):
            self._wget_id(strain_id["ptt"], filename[1], strain_id, files)
        if (filename[1] in os.listdir(files["id_list"])) or (
                "all" in querys):
            if (filename[1] in os.listdir(files["id_list"])):
                id_h = open(os.path.join(files["id_list"], filename[1]), "r")
                for row_i in csv.reader(id_h, delimiter="\t"):
                    if row_a == row_i[0]:
                        prefername = row_i[3]
                id_h.close()
        return prefername

    def _print_title(self, out, id_file, id_folder):
        id_h = open(os.path.join(id_folder, id_file), "r")
        prefername = id_file
        for row_i in csv.reader(id_h, delimiter="\t"):
            prefername = row_i[3]
        id_h.close()
        if prefername not in self.ref_tags.keys():
            locus = id_file
        else:
            locus = self.ref_tags[prefername]
        out.write("Interaction of {0} | {1}\n".format(
            locus, prefername))
        out.write("Genome\tItem_id_a\tItem_id_b\tMode\tAction\ta_is_acting\t"
                  "STRING_action_score\tPubmed_id\tPubmed_score\n")

    def _get_pubmed(self, row, strain_id, mode, actor, id_file, first_output,
                    ptt, files, paths, args_ppi):
        prefer1 = self._get_prefer_name(row[0], strain_id,
                                        files, args_ppi.querys)
        prefer2 = self._get_prefer_name(row[1], strain_id,
                                        files, args_ppi.querys)
        if (len(prefer1) > 0) and (len(prefer2) > 0):
            if args_ppi.no_specific:
                pubmed_source = (
                    "http://www.ncbi.nlm.nih.gov/CBBresearch/"
                    "Wilbur/IRET/PIE/getppi.cgi?term={0}+{1}").format(
                        prefer1, prefer2)
                self._run_wget(pubmed_source, self.tmp_files["nospecific"],
                               files["pubmed_log"])
            strain_id["pie"] = "+".join(strain_id["pie"].split(" "))
            pubmed_source = (
                "http://www.ncbi.nlm.nih.gov/CBBresearch/Wilbur"
                "/IRET/PIE/getppi.cgi?term={0}+{1}+{2}").format(
                    prefer1, prefer2, strain_id["pie"])
            self._run_wget(pubmed_source, self.tmp_files["specific"],
                           files["pubmed_log"])
            row[2] = mode
            row[4] = actor
            row[0] = prefer1
            row[1] = prefer2
            self._merge_information(
                first_output, self.tmp_files["specific"],
                files["all_specific"], files["best_specific"], row,
                args_ppi.score, id_file, files["id_list"], "specific",
                os.path.join(paths["all"], self.with_strain),
                os.path.join(paths["best"], self.with_strain), ptt)
            if args_ppi.no_specific:
                self._merge_information(
                     first_output, self.tmp_files["nospecific"],
                     files["all_nospecific"], files["best_nospecific"], row,
                     args_ppi.score, id_file, files["id_list"], "nospecific",
                     os.path.join(paths["all"], self.without_strain),
                     os.path.join(paths["best"], self.without_strain), ptt)

    def _print_single_file(self, out_single, row_a, ptt, row):
        if row == "NA":
            out_single.write("\t".join(
                             [ptt, "\t".join(row_a), "NA", "NA"]) + "\n")
        else:
            out_single.write("\t".join(
                             [ptt, "\t".join(row_a), "\t".join(row)]) + "\n")

    def _merge_information(self, first_output, filename, out_all, out_best,
                           row_a, score, id_file, id_folder, file_type,
                           all_folder, best_folder, ptt):
        if os.path.getsize(filename) != 0:
            f_h = open(filename, "r")
            out_all_single = open(os.path.join(
                all_folder, ptt, "_".join([row_a[0], row_a[1] + ".csv"])), "w")
            out_best_single = open(os.path.join(
                best_folder, ptt,
                "_".join([row_a[0], row_a[1] + ".csv"])), "w")
            self._print_title(out_all_single, id_file, id_folder)
            self._print_title(out_best_single, id_file, id_folder)
            detect = False
            for row in csv.reader(f_h, delimiter="\t"):
                self._print_single_file(out_all_single, row_a, ptt, row)
                if first_output["_".join([file_type, "all"])]:
                    first_output["_".join([file_type, "all"])] = False
                    self._print_title(out_all, id_file, id_folder)
                out_all.write("\t".join([ptt, "\t".join(row_a),
                                         "\t".join(row)]) + "\n")
                if (float(row[1]) >= score):
                    detect = True
                    self._print_single_file(out_best_single, row_a, ptt, row)
                    if first_output["_".join([file_type, "best"])]:
                        first_output["_".join([file_type, "best"])] = False
                        self._print_title(out_best, id_file, id_folder)
                    out_best.write("\t".join([ptt, "\t".join(row_a),
                                              "\t".join(row)]) + "\n")
            f_h.close()
            if not detect:
                os.remove(os.path.join(best_folder, ptt,
                          "_".join([row_a[0], row_a[1] + ".csv"])))
            out_all_single.close()
            out_best_single.close()
        else:
            out_all_single = open(os.path.join(
                all_folder, ptt, "_".join([row_a[0], row_a[1] + ".csv"])), "w")
            self._print_title(out_all_single, id_file, id_folder)
            self._print_single_file(out_all_single, row_a, ptt, "NA")
            if first_output["_".join([file_type, "all"])]:
                first_output["_".join([file_type, "all"])] = False
                self._print_title(out_all, id_file, id_folder)
            out_all.write("\t".join([ptt, "\t".join(row_a),
                                     "NA", "NA"]) + "\n")
            out_all_single.close()

    def _detect_protein(self, strain_id, args_ppi):
        fh = open(os.path.join(args_ppi.ptts, strain_id["file"]), "r")
        genes = []
        for row in csv.reader(fh, delimiter="\t"):
            if (len(row) == 1) and ("-" in row[0]) and (".." in row[0]):
                name = (row[0].split("-"))[0].strip().split(",")[0].strip()
            if ("all" in args_ppi.querys):
                if (len(row) > 1) and (row[0] != "Location"):
                    genes.append({"strain": name, "locus_tag": row[5],
                                  "gene": row[4]})
            else:
                for query in args_ppi.querys:
                    datas = query.split(":")
                    strain = datas[0]
                    start = datas[1]
                    end = datas[2]
                    strand = datas[3]
                    if (len(row) > 1) and (row[0] != "Location") and (
                            name == strain) and (
                            start == row[0].split("..")[0]) and (
                            end == row[0].split("..")[1]) and (
                            strand == row[1]):
                        genes.append({"strain": name, "locus_tag": row[5],
                                      "gene": row[4]})
        fh.close()
        return genes

    def _setup_nospecific(self, paths, strain_id, files):
        self._make_subfolder(
            paths["all"], self.without_strain, strain_id["ptt"])
        self._make_subfolder(
            paths["best"], self.without_strain, strain_id["ptt"])
        self._make_subfolder(
            paths["fig"], self.without_strain, strain_id["ptt"])
        filename_nostrain = "_".join([strain_id["file"].replace(".ptt", ""),
                                      self.without_strain + ".csv"])
        files["all_nospecific"] = open(os.path.join(paths["all"],
                                       filename_nostrain), "w")
        files["best_nospecific"] = open(os.path.join(paths["best"],
                                        filename_nostrain), "w")

    def _setup_folder_and_read_file(self, strain_id, pre_file,
                                    files, paths, args_ppi):
        if strain_id["file"].endswith(".ptt"):
            if strain_id["file"] != pre_file:
                self.helper.check_make_folder(
                     "_".join([self.tmp_id, strain_id["file"]]))
                paths["all"] = os.path.join(
                     self.all_result, strain_id["file"][:-4])
                paths["best"] = os.path.join(
                     self.best_result, strain_id["file"][:-4])
                paths["fig"] = os.path.join(
                     self.fig, strain_id["file"][:-4])
                self.helper.check_make_folder(
                     os.path.join(self.all_result, strain_id["file"][:-4]))
                self.helper.check_make_folder(
                     os.path.join(self.best_result, strain_id["file"][:-4]))
                self.helper.check_make_folder(
                     os.path.join(self.fig, strain_id["file"][:-4]))
                self._make_subfolder(
                     paths["all"], self.with_strain, strain_id["ptt"])
                self._make_subfolder(
                     paths["best"], self.with_strain, strain_id["ptt"])
                self._make_subfolder(
                     paths["fig"], self.with_strain, strain_id["ptt"])
                filename_strain = "_".join(
                     [strain_id["file"].replace(".ptt", ""),
                      self.with_strain + ".csv"])
                files["all_specific"] = open(os.path.join(
                                        paths["all"], filename_strain), "w")
                files["best_specific"] = open(os.path.join(
                                         paths["best"], filename_strain), "w")
                if args_ppi.no_specific:
                    self._setup_nospecific(paths, strain_id, files)
                files["id_list"] = "_".join([self.tmp_id, strain_id["file"]])
                files["id_log"] = open(os.path.join(files["id_list"],
                                       self.tmp_files["log"]), "w")
                files["action_log"] = open(os.path.join(args_ppi.out_folder,
                                           self.tmp_files["action"]), "w")
                files["pubmed_log"] = open(os.path.join(args_ppi.out_folder,
                                           self.tmp_files["pubmed"]), "w")
                pre_file = strain_id["file"]
                if strain_id["file"] in os.listdir(args_ppi.ptts):
                    genes = self._detect_protein(strain_id, args_ppi)
            else:
                self._make_folder_no_exist(os.path.join(paths["all"],
                                           self.with_strain), strain_id["ptt"])
                self._make_folder_no_exist(os.path.join(paths["best"],
                                           self.with_strain), strain_id["ptt"])
                if args_ppi.no_specific:
                    self._make_folder_no_exist(
                        os.path.join(paths["all"], self.without_strain),
                        strain_id["ptt"])
                    self._make_folder_no_exist(
                        os.path.join(paths["best"], self.without_strain),
                        strain_id["ptt"])
        else:
            print("Error: Wrong .ptt file!")
            sys.exit()
        return genes

    def _wget_actions(self, files, id_file, strain_id, out_folder):
        detect = False
        t_h = open(os.path.join(files["id_list"], id_file), "r")
        print("Retrieving STRING actions for {0} of {1} -- {2}".format(
              id_file, strain_id["string"], strain_id["file"]))
        for row in csv.reader(t_h, delimiter="\t"):
            if row[0].startswith("stringId"):
                continue
            else:
                detect = True
                if row[1] == strain_id["string"]:
                    action_source = ("http://string-db.org/api/tsv/actions?"
                                     "identifier={0}&species={1}").format(
                                     row[0], row[1])
                    self._run_wget(
                        action_source, self.tmp_files["wget_action"],
                        files["action_log"])
        t_h.close()
        if not detect:
            print("Warning: " + id_file + " can not be found in STRING!")
        return detect

    def _retrieve_actions(self, files, strain_id, paths, args_ppi):
        '''get the interaction of proteins'''
        for id_file in os.listdir(files["id_list"]):
            if id_file != self.tmp_files["log"]:
                detect_id = self._wget_actions(files, id_file, strain_id,
                                               args_ppi.out_folder)
                if detect_id:
                    a_h = open(self.tmp_files["wget_action"], "r")
                    pre_row = []
                    first = True
                    detect = False
                    first_output = {"specific_all": True,
                                    "specific_best": True,
                                    "nospecific_all": True,
                                    "nospecific_best": True}
                    print("Retrieving Pubmed IDs for {0} of {1} -- {2}".format(
                           id_file, strain_id["string"], strain_id["file"]))
                    for row_a in csv.reader(a_h, delimiter="\t"):
                        if row_a == []:
                            print("No interaction can be detected")
                            break
                        if row_a[0].startswith("item_id_a"):
                            continue
                        else:
                            detect = True
                            if first:
                                first = False
                                mode = row_a[2]
                                actor = row_a[4]
                            else:
                                if (row_a[0] != pre_row[0]) or (
                                        row_a[1] != pre_row[1]):
                                    self._get_pubmed(
                                        pre_row, strain_id, mode, actor,
                                        id_file, first_output,
                                        strain_id["ptt"], files, paths,
                                        args_ppi)
                                    mode = row_a[2]
                                    actor = row_a[4]
                                else:
                                    mode = mode + ";" + row_a[2]
                                    actor = actor + ";" + row_a[4]
                            pre_row = row_a
                    if detect:
                        detect = False
                        self._get_pubmed(
                            row_a, strain_id, mode, actor, id_file,
                            first_output, strain_id["ptt"], files,
                            paths, args_ppi)
        if detect_id:
            a_h.close()

    def _plot(self, args_ppi, files):
        if args_ppi.no_specific:
            files["all_nospecific"].close()
            files["best_nospecific"].close()
        files["all_specific"].close()
        files["best_specific"].close()
        for folder in os.listdir(self.all_result):
            if folder in os.listdir(self.fig):
                print("Plotting {0}".format(folder))
                plot_ppi(os.path.join(self.all_result, folder,
                         "_".join([folder, self.with_strain + ".csv"])),
                         args_ppi.score, os.path.join(self.fig, folder,
                         self.with_strain), args_ppi.size)
                if args_ppi.no_specific:
                    plot_ppi(os.path.join(self.all_result, folder,
                             "_".join([folder, self.without_strain + ".csv"])),
                             args_ppi.score,
                             os.path.join(self.fig, folder,
                             self.without_strain), args_ppi.size)

    def _remove_tmps(self, args_ppi):
        self.helper.remove_all_content(os.path.join(args_ppi.out_folder),
                                       "tmp", "file")
        self.helper.remove_all_content(os.path.join(args_ppi.out_folder),
                                       "tmp", "dir")
        for file_ in os.listdir(args_ppi.ptts):
            if file_.startswith("PPI_"):
                os.remove(os.path.join(args_ppi.ptts, file_))
        self.helper.remove_all_content(os.path.join(args_ppi.out_folder),
                                       "temp", "dir")

    def check_query(self, args_ppi):
        for query in args_ppi.querys:
            detect = False
            datas = query.split(":")
            for gff in os.listdir(args_ppi.ptts):
                gff_f = open(os.path.join(args_ppi.ptts, gff), "r")
                for entry in Gff3Parser().entries(gff_f):
                    if (entry.seq_id == datas[0]) and (
                            entry.start == int(datas[1])) and (
                            entry.end == int(datas[2])) and (
                            entry.strand == datas[3]):
                        detect = True
                        break
                if not detect:
                    print("Error: Some of the query proteins do not exist!")
                    sys.exit()

    def retrieve_ppi_network(self, args_ppi):
        '''retrieve PPI from STRING with PIE and draw network'''
        strain_ids = []
        paths = {}
        files = {}
        self.check_query(args_ppi)
        for strain in args_ppi.strains:
            datas = strain.split(":")
            ptt_file = "PPI_" + datas[0].replace(".gff", ".ptt")
            rnt_file = "PPI_" + datas[0].replace(".gff", ".rnt")
            self.converter.convert_gff2rntptt(
                           os.path.join(args_ppi.ptts, datas[0]),
                           "0", os.path.join(args_ppi.ptts, ptt_file),
                           os.path.join(args_ppi.ptts, rnt_file), None, None)
            strain_ids.append({"file": ptt_file,
                               "ptt": datas[1],
                               "string": datas[2],
                               "pie": datas[3]})
        strain_ids.sort(key=lambda x: x["file"])
        pre_file = ""
        for strain_id in strain_ids:
            genes = self._setup_folder_and_read_file(strain_id, pre_file,
                                                     files, paths, args_ppi)
            s_h = open(args_ppi.species, "r")
            for row in csv.reader(s_h, delimiter="\t"):
                if row[0] != "##":
                    if row[0] == strain_id["string"]:
                        break
                    elif row[2] == strain_id["string"]:
                        strain_id["string"] = row[0]
                        break
                    elif row[3] == strain_id["string"]:
                        strain_id["string"] = row[0]
                        break
            self._retrieve_id(strain_id, genes, files)
            self._retrieve_actions(files, strain_id, paths, args_ppi)
        self._plot(args_ppi, files)
#        self._remove_tmps(args_ppi)



import os
import sys
import copy
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.lib_reader import read_wig, read_libs
from annogesiclib.plot_coverage_table import plot_table


def read_data(gff, features):
    gffs = {}
    stats = {}
    outs = {}
    for entry in Gff3Parser().entries(open(gff)):
        for feature in features:
            if feature not in gffs.keys():
                gffs[feature] = []
                stats[feature] = {}
                outs[feature] = {"all": [], "least_one": [], "none": []}
            if entry.feature == feature:
                gffs[feature].append(entry)
    for feature in gffs.keys():
        gffs[feature] = sorted(gffs[feature], key=lambda k: (
            k.seq_id, k.start, k.end, k.strand))
    return gffs, stats, outs


def set_cutoff(cond, percent_tex, percent_frag, detects, gff):
    if ("tex" in cond) or ("notex" in cond):
        cutoffs = percent_tex.split("_")
    elif ("frag" in cond):
        cutoffs = percent_frag.split("_")
    if cutoffs[0] == "p":
        cutoff_percent = float(cutoffs[1])
        diff = detects["express"] / (gff.end - gff.start + 1)
    elif cutoffs[0] == "n":
        cutoff_percent = float(cutoffs[1])
        diff = detects["express"]
    elif cutoffs[0] == "all":
        cutoff_percent = 0
        diff = detects["express"]
    else:
        print("Error: Please assign the valid cutoff_overlap!!")
    return diff, cutoff_percent


def detect_express(wigs, gff, cutoff_coverage, detects, percent_tex,
                   percent_frag, texs, cond, tex_notex, track, plots,
                   cover_type, name):
    total = 0
    high = 0
    for wig in wigs[(gff.start - 1): gff.end]:
        total = wig["coverage"] + total
        if wig["coverage"] > high:
            high = wig["coverage"]
        if wig["coverage"] >= cutoff_coverage:
            detects["express"] += 1
    if cover_type == "average":
        plots[cond][name] = float(total) / float(gff.end - gff.start + 1)
    elif cover_type == "high":
        plots[cond][name] = high
    else:
        print("Error: The coverage_type is not correct!!!")
        sys.exit()
    diff, cutoff_percent = set_cutoff(cond, percent_tex, percent_frag,
                                      detects, gff)
    if (diff >= float(cutoff_percent)) and (diff != 0):
        if ("tex" in cond) or ("notex" in cond):
            for key, num in texs.items():
                if track in key:
                    texs[key] += 1
                if texs[key] == tex_notex:
                    detects["track"] += 1
        elif "frag" in cond:
            detects["track"] += 1


def compare_wigs(wigs, gff, tex_notex, template_texs, replicates, stats,
                 outs, plots, cover_type, cutoff_coverage, percent_tex,
                 percent_frag):
    detects = {"cond": 0, "track": 0, "import": False, "express": 0}
    texs = copy.deepcopy(template_texs)
    for strain, conds in wigs.items():
        if gff.seq_id == strain:
            detects["cond"] = 0
            num_conds = 0
            for cond, tracks in conds.items():
                num_conds += 1
                plots[cond] = {}
                if cond not in stats[strain].keys():
                    stats[strain][cond] = 0
                if cond not in stats["total"].keys():
                    stats["total"][cond] = 0
                if cond not in outs.keys():
                    outs[cond] = []
                detects["track"] = 0
                for track, wigs in tracks.items():
                    name = track
                    plots[cond][name] = 0
                    detects["express"] = 0
                    detect_express(wigs, gff, cutoff_coverage, detects,
                                   percent_tex, percent_frag, texs, cond,
                                   tex_notex, track, plots, cover_type, name)
                if ("tex" in cond) or ("notex" in cond):
                    if detects["track"] >= replicates["tex"]:
                        detects["import"] = True
                elif ("frag" in cond):
                    if detects["track"] >= replicates["frag"]:
                        detects["import"] = True
                if detects["import"]:
                    detects["import"] = False
                    stats["total"][cond] += 1
                    stats[strain][cond] += 1
                    outs[cond].append(gff)
                    detects["cond"] += 1
            if detects["cond"] == 0:
                stats["total"]["none"] += 1
                stats[strain]["none"] += 1
                outs["none"].append(gff)
            if detects["cond"] == num_conds:
                stats["total"]["all"] += 1
                stats[strain]["all"] += 1
                outs["all"].append(gff)
            if (detects["cond"] <= num_conds) and (
                  detects["cond"] > 0):
                stats["total"]["least_one"] += 1
                stats[strain]["least_one"] += 1
                outs["least_one"].append(gff)


def print_stat(out, stats):
    out.write("\t".join(["total input:", str(stats["total"])]) + "\n")
    for cond, num in stats.items():
        if cond == "least_one":
            tag = "expression at lease one condition:"
        elif cond == "all":
            tag = "expression at all conditions:"
        elif cond == "none":
            tag = "no expression:"
        else:
            tag = "condition " + cond + ":"
        if cond != "total":
            per = "(" + str(float(num) / float(stats["total"])) + ")"
            out.write("\t".join([tag, " ".join([str(num), per])]) + "\n")


def output_stat(stats, stat_folder, prefix):
    for feature, strains in stats.items():
        out = open(os.path.join(stat_folder,
                   "_".join([prefix, feature + ".csv"])), "w")
        if len(strains.keys()) > 2:
            out.write("All strain:\n")
            print_stat(out, strains["total"])
        for strain, stat in strains.items():
            if strain != "total":
                out.write(strain + ":\n")
                print_stat(out, stat)
        out.close()


def output_gff(outs, out_gff_folder, prefix):
    for feature, conds in outs.items():
        for cond, gffs in conds.items():
            if cond == "least_one":
                out = open(os.path.join(
                    out_gff_folder, "_".join([
                        prefix, feature, "at_least_one_lib.gff"])), "w")
            elif cond == "all":
                out = open(os.path.join(
                    out_gff_folder, "_".join([
                        prefix, feature, "all_libs.gff"])), "w")
            elif cond == "none":
                out = open(os.path.join(
                    out_gff_folder, "_".join([
                        prefix, feature, "no_express.gff"])), "w")
            else:
                out = open(os.path.join(
                    out_gff_folder, "_".join([
                        prefix, feature, cond + ".gff"])), "w")
            out.write("##gff-version 3\n")
            for gff in gffs:
                out.write(gff.info + "\n")
            out.close()


def deal_repeat_tag(gff, plots, feature, repeat, tag, tags):
    if (gff.attributes[tag] in tags) and (
            gff.attributes[tag] not in repeat.keys()):
        plots[feature].append({gff.attributes[tag] + "_2": {}})
        repeat[gff.attributes[tag]] = 2
        name = gff.attributes[tag] + "_2"
    elif (gff.attributes[tag] in tags) and (
            gff.attributes[tag] in repeat.keys()):
            plots[feature].append({"_".join([gff.attributes[tag],
                                   str(repeat[gff.attributes[tag]] + 1)]): {}})
            name = "_".join([gff.attributes[tag],
                             str(repeat[gff.attributes[tag]] + 1)])
            repeat[gff.attributes[tag]] += 1
    else:
        plots[feature].append({gff.attributes[tag]: {}})
        name = gff.attributes[tag]
    return name


def get_name(plots, gff, feature, repeat, tags):
    name = "".join([gff.feature, ":", str(gff.start),
                    "-", str(gff.end), "_", gff.strand])
    if feature == "gene":
        if "locus_tag" in gff.attributes.keys():
            name = deal_repeat_tag(gff, plots, feature,
                                   repeat, "locus_tag", tags)
        else:
            plots[feature].append({name: {}})
    elif feature == "CDS":
        if "locus_tag" in gff.attributes.keys():
            name = deal_repeat_tag(gff, plots, feature,
                                   repeat, "locus_tag", tags)
        elif "protein_id" in gff.attributes.keys():
            name = deal_repeat_tag(gff, plots, feature,
                                   repeat, "protein_id", tags)
        else:
            plots[feature].append({name: {}})
    else:
        plots[feature].append({name: {}})
    tags.append(name)
    return name


def plot(plots, stat_folder, max_color, min_color, cover_type):
    for feature in plots:
        plot_table(plots[feature], max_color, min_color,
                   os.path.join(stat_folder, "_".join([
                       feature, cover_type, "express_analysis.png"])))


def gene_expression(input_libs, gff_folder, percent_tex, percent_frag,
                    wig_f_file, wig_r_file, features, wigs, cutoff_coverage,
                    tex_notex, replicates, stat_folder, out_gff_folder,
                    cover_type, max_color, min_color):
    print("Loading wiggle file...")
    libs, texs = read_libs(input_libs, wigs)
    wig_fs = read_wig(wig_f_file, "+", libs)
    wig_rs = read_wig(wig_r_file, "-", libs)
    plots = {}
    repeat = {}
    for gff in os.listdir(gff_folder):
        if gff.endswith(".gff"):
            prefix = gff.replace(".gff", "")
            print("Computing " + prefix)
            gff_list, stats, outs = read_data(os.path.join(gff_folder, gff),
                                              features)
            for feature, gffs in gff_list.items():
                plots[feature] = []
                repeat[feature] = {}
                tags = []
                stats[feature]["total"] = {"total": 0, "least_one": 0,
                                           "all": 0, "none": 0}
                num = 0
                for gff in gffs:
                    if gff.seq_id not in stats[feature].keys():
                        stats[feature][gff.seq_id] = {
                                "total": 0, "least_one": 0,
                                "all": 0, "none": 0}
                    stats[feature]["total"]["total"] += 1
                    stats[feature][gff.seq_id]["total"] += 1
                    name = get_name(plots, gff, feature, repeat[feature], tags)
                    if gff.strand == "+":
                        compare_wigs(
                                wig_fs, gff, tex_notex, texs, replicates,
                                stats[feature], outs[feature],
                                plots[feature][num][name], cover_type,
                                cutoff_coverage, percent_tex, percent_frag)
                    elif gff.strand == "-":
                        compare_wigs(
                                wig_rs, gff, tex_notex, texs, replicates,
                                stats[feature], outs[feature],
                                plots[feature][num][name], cover_type,
                                cutoff_coverage, percent_tex, percent_frag)
                    num += 1
            output_stat(stats, stat_folder, prefix)
            output_gff(outs, out_gff_folder, prefix)
    plot(plots, stat_folder, max_color, min_color, cover_type)



import csv
from annogesiclib.gff3 import Gff3Parser


def plus_num(nums, strain, feature):
    nums[strain][feature] += 1
    nums["total"][feature] += 1


def print_percent(out, total, fract, name):
    if total != 0:
        out.write("\t\t(percentage of total {0}terminators = {1})\n".format(
                  name, (fract / total)))
    else:
        out.write("\t\t(percentage of total {0}terminators = 0.0)\n".format(
                  name))


def print_express(out, total, fract, name):
    if total != 0:
        out.write("\t\t(percentage of total {0}terminators which have "
                  "gene expression = {1})\n".format(name, (fract / total)))
    else:
        out.write("\t\t(percentage of total {0}terminators which have "
                  "gene expression = 0.0)\n".format(name))


def print_decrease(out, total, fract, name):
    if total != 0:
        out.write("\t\t(percentage of total {0}terminators "
                  "which have dramatic coverage decreasing = {1})\n".format(
                      name, (fract / total)))
    else:
        out.write("\t\t(percentage of total {0}terminators "
                  "which have dramatic coverage decreasing = 0.0)\n".format(
                      name))


def print_method(nums, method_name, method, express, detect, only, out):
    out.write(method_name + "\n")
    out.write("\tTotal {0} terminators = {1}\n".format(
              method_name, nums[method] + nums["frhp"]))
    print_percent(out, float(nums["total"]),
                  float(nums[method] + nums["frhp"]), "")
    out.write("\tTotal terminators which only can be "
              "detected in {0} = {1}\n".format(method_name, nums[method]))
    print_percent(out, float(nums["total"]), float(nums[method]), "")
    print_percent(out, float(nums[method] + nums["frhp"]),
                  float(nums[method]), method_name + " ")
    out.write("\tTotal {0} terminators which located in gene "
              "expression region = {1}\n".format(method_name, nums[express]))
    print_percent(out, float(nums["total"]), float(nums[express]), "")
    print_percent(out, float(nums[method] + nums["frhp"]),
                  float(nums[express]), method_name + " ")
    out.write("\tTotal {0} terminators which have dramatic coverage "
              "decreasing = {1}\n".format(method_name, nums[detect]))
    print_percent(out, float(nums["total"]), float(nums[detect]), "")
    print_percent(out, float(nums[method] + nums["frhp"]),
                  float(nums[detect]), method_name + " ")
    print_express(out, float(nums["total_ex"]), float(nums[detect]), "")
    print_express(out, float(nums[express]), float(nums[detect]),
                  method_name + " ")
    out.write("\tTotal terminators which have dramatic coverage decreasing"
              "(unique in {0}) = {1}\n".format(method_name, nums[only]))
    print_decrease(out, float(nums["total_de"]), float(nums[only]), "")
    print_decrease(out, float(nums[detect]), float(nums[only]),
                   method_name + " ")
    out.write("\n")


def print_intersection_number(out, nums, type_):
    print_percent(out, float(nums["total"]), float(nums[type_]), "")
    print_percent(out, float(nums["fr"]), float(nums[type_]), "method_1 ")
    print_percent(out, float(nums["hp"]), float(nums[type_]), "method_2 ")


def print_intersection_express(out, nums, type_):
    print_express(out, float(nums["total_ex"]), float(nums[type_]), "")
    print_express(out, float(nums["ex_fr"]), float(nums[type_]), "method_1 ")
    print_express(out, float(nums["ex_hp"]), float(nums[type_]), "method_2 ")


def print_file(nums, out, strain):
    out.write(strain + ":\n")
    out.write("Combine two methods:\n")
    out.write("\tTotal terminators = {0}\n".format(nums["total"]))
    out.write("\tTotal terminators which located in gene expression "
              "region = {0}\n".format(nums["total_ex"]))
    print_percent(out, float(nums["total"]), float(nums["total_ex"]), "")
    out.write("\tTotal terminators which have dramatic coverage "
              "decreasing = {0}\n".format(nums["total_de"]))
    print_percent(out, float(nums["total"]), float(nums["total_de"]), "")
    print_express(out, float(nums["total_ex"]), float(nums["total_de"]), "")
    out.write("\n")
    print_method(nums, "method_1", "fr", "ex_fr", "de_fr", "only_de_fr", out)
    print_method(nums, "method_2", "hp", "ex_hp", "de_hp", "only_de_hp", out)
    out.write("intersection two methods:\n")
    out.write("\tTotal terminators which overlap with "
              "two methods = {0}\n".format(nums["frhp"]))
    print_intersection_number(out, nums, "frhp")
    out.write("\tTotal overlaped terminators which located in "
              "gene expression region = {0}\n".format(nums["ex_frhp"]))
    print_intersection_number(out, nums, "ex_frhp")
    print_intersection_express(out, nums, "ex_frhp")
    out.write("\tTotal overlaped terminators which have dramatic "
              "coverage decreasing = {0}\n".format(nums["de_frhp"]))
    print_intersection_number(out, nums, "de_frhp")
    print_intersection_express(out, nums, "de_frhp")
    print_express(out, float(nums["total_de"]), float(nums["de_frhp"]), "")
    print_express(out, float(nums["de_fr"]),
                  float(nums["de_frhp"]), "method_1 ")
    print_express(out, float(nums["de_hp"]),
                  float(nums["de_frhp"]), "method_2 ")


def classify_terms(terms, nums, out_d, out_e, out_n, pre_strain):
    for term in terms:
        if term.seq_id != pre_strain:
            pre_strain = term.seq_id
            strain = term.seq_id
            nums[strain] = {
                    "fr": 0, "hp": 0, "frhp": 0, "ex_fr": 0, "ex_hp": 0,
                    "ex_frhp": 0, "de_fr": 0, "de_hp": 0, "de_frhp": 0,
                    "total": 0, "total_de": 0, "total_ex": 0, "only_de_fr": 0,
                    "only_de_hp": 0, "only_ex_fr": 0, "only_ex_hp": 0,
                    "de_frhp": 0, "ex_frhp": 0}
        if term.attributes["coverage_decrease"] == "True":
            out_d.write(term.info + "\n")
        if term.attributes["express"] == "True":
            out_e.write(term.info + "\n")
        if term.attributes["express"] != "True":
            out_n.write(term.info + "\n")
        if term.attributes["method"] == "gene_converged":
            plus_num(nums, strain, "total")
            plus_num(nums, strain, "fr")
            if term.attributes["coverage_decrease"] == "True":
                plus_num(nums, strain, "de_fr")
                plus_num(nums, strain, "only_de_fr")
                plus_num(nums, strain, "total_de")
            if term.attributes["express"] == "True":
                plus_num(nums, strain, "ex_fr")
                plus_num(nums, strain, "only_ex_fr")
                plus_num(nums, strain, "total_ex")
        elif term.attributes["method"] == "TransTermHP":
            plus_num(nums, strain,  "total")
            plus_num(nums, strain,  "hp")
            if term.attributes["coverage_decrease"] == "True":
                plus_num(nums, strain, "de_hp")
                plus_num(nums, strain, "only_de_hp")
                plus_num(nums, strain, "total_de")
            if term.attributes["express"] == "True":
                plus_num(nums, strain, "ex_hp")
                plus_num(nums, strain, "only_ex_hp")
                plus_num(nums, strain, "total_ex")
        elif term.attributes["method"] == "gene_converged,TransTermHP":
            plus_num(nums, strain, "total")
            plus_num(nums, strain, "frhp")
            if term.attributes["coverage_decrease"] == "True":
                plus_num(nums, strain, "de_frhp")
                plus_num(nums, strain, "de_fr")
                plus_num(nums, strain, "de_hp")
                plus_num(nums, strain, "total_de")
            if term.attributes["express"] == "True":
                plus_num(nums, strain, "ex_frhp")
                plus_num(nums, strain, "ex_fr")
                plus_num(nums, strain, "ex_hp")
                plus_num(nums, strain, "total_ex")


def check_repeat(checks, strain, start, end, strand):
    detect = False
    try:
        term = {"strain": strain, "start": int(start),
                "strand": strand, "end": int(end)}
        if len(checks) == 0:
            checks.append(term)
            detect = True
        else:
            if term not in checks:
                detect = True
                checks.append(term)
        return detect
    except ValueError:
        return detect


def stat_term(term_gff, term_table, stat, output_decrease,
              output_expression, output_non):
    terms = []
    nums = {}
    nums["total"] = {
            "fr": 0, "hp": 0, "frhp": 0, "ex_fr": 0, "ex_hp": 0, "ex_frhp": 0,
            "de_fr": 0, "de_hp": 0, "de_frhp": 0, "total": 0, "total_de": 0,
            "total_ex": 0, "only_de_fr": 0, "only_de_hp": 0, "only_ex_fr": 0,
            "only_ex_hp": 0, "de_frhp": 0, "ex_frhp": 0}
    pre_strain = ""
    out_te = open(output_expression + ".csv", "w")
    out_td = open(output_decrease + ".csv", "w")
    out_tn = open(output_non + ".csv", "w")
    fh = open(term_table, "r")
    out_tn.write("\t".join(["Genome", "Name", "Start", "End", "Strand",
                            "Detect", "Associated_gene", "Associated_transcript",
                            "Coverage_decrease", "Coverage_detail"]) + "\n")
    gh = open(term_gff)
    checks = []
    for entry in Gff3Parser().entries(gh):
        detect = check_repeat(checks, entry.seq_id, entry.start,
                              entry.end, entry.strand)
        if detect:
            terms.append(entry)
    checks = []
    for row in csv.reader(fh, delimiter="\t"):
        detect = check_repeat(checks, row[0], row[2], row[3], row[4])
        if detect:
            if (row[-1] != "NA") and (row[-1] != "No_coverage_decreasing"):
                out_td.write("\t".join(row) + "\n")
                out_te.write("\t".join(row) + "\n")
            if (row[-1] == "No_coverage_decreasing"):
                out_te.write("\t".join(row) + "\n")
            if (row[-1] == "NA"):
                out_tn.write("\t".join(row) + "\n")
    out = open(stat, "w")
    out_e = open(output_expression + ".gff", "w")
    out_d = open(output_decrease + ".gff", "w")
    out_n = open(output_non + ".gff", "w")
    out_e.write("##gff-version 3\n")
    out_d.write("##gff-version 3\n")
    out_n.write("##gff-version 3\n")
    classify_terms(terms, nums, out_d, out_e, out_n, pre_strain)
    out.write("method_1 is searching the gene converged region.\n")
    out.write("method_2 is TransTermHP.\n")
    if len(nums) > 2:
        print_file(nums["total"], out, "All genome")
    else:
        for strain, datas in nums.items():
            if strain != "total":
                print_file(datas, out, strain)
    out_te.close()
    out_td.close()
    out_tn.close()
    out.close()
    out_e.close()
    out_d.close()
    out_n.close()
    fh.close()
    gh.close()



from annogesiclib.gff3 import Gff3Parser


def print_stat(feature, num, out):
    if num["_".join(["all", feature])] != 0:
        out.write("The number of {0} which is start "
                  "from TSS: {1} ({2})\n".format(
                      feature, num[feature],
                      float(num[feature]) / float(
                          num["_".join(["all", feature])])))
    else:
        out.write("The number of {0} which is start "
                  "from TSS: {1} ({2})\n".format(
                      feature, num[feature], "NA"))


def read_gff(gff_file, tss_file):
    tsss = []
    gffs = []
    gff_parser = Gff3Parser()
    fh = open(gff_file)
    for gff in gff_parser.entries(fh):
        gffs.append(gff)
    gffs = sorted(gffs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    fh.close()
    tss_f = open(tss_file, "r")
    for tss in gff_parser.entries(tss_f):
        tsss.append(tss)
    tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    tss_f.close()
    return gffs, tsss


def compare_tss(tsss, gff, utr_length, num_all, num_strain, program):
    detect = False
    for tss in tsss:
        length = utr_length
        if (gff.feature == "CDS") or (
                gff.feature == "rRNA") or (
                gff.feature == "tRNA"):
            if (gff.seq_id == tss.seq_id) and (
                    gff.start < tss.start) and (
                    gff.strand == "+") and (tss.strand == "+"):
                break
            elif (gff.seq_id == tss.seq_id) and (
                    gff.end < tss.start - utr_length) and (
                    gff.strand == "-") and (tss.strand == "-"):
                break
            if (gff.seq_id == tss.seq_id) and (
                    gff.strand == "+") and (tss.strand == "+") and (
                    gff.start - tss.start <= utr_length) and (
                    gff.start - tss.start >= 0):
                detect = True
                if (gff.start - tss.start) <= length:
                    start = tss
                    length = (gff.start - tss.start)
            elif (gff.seq_id == tss.seq_id) and (
                    gff.strand == "-") and (tss.strand == "-") and (
                    tss.start - gff.end <= utr_length) and (
                    tss.start - gff.end >= 0):
                detect = True
                if (tss.start - gff.end) <= length:
                    start = tss
                    length = (tss.start - gff.end)
    if program == "tss":
        type_ = "TSS"
    elif program == "processing":
        type_ = "Cleavage"
    if detect:
        detect = False
        gff.attributes["start_" + type_] = (
                type_ + "_" + str(start.start) + start.strand)
        if gff.feature == "CDS":
            num_all["cds"] += 1
            num_strain[gff.seq_id]["cds"] += 1
        elif gff.feature == "tRNA":
            num_all["tRNA"] += 1
            num_strain[gff.seq_id]["tRNA"] += 1
        elif gff.feature == "rRNA":
            num_all["rRNA"] += 1
            num_strain[gff.seq_id]["rRNA"] += 1


def print_file(gffs, out_cds_file, stat_file, num_all, num_strain):
    out_cds = open(out_cds_file, "w")
    out_cds.write("##gff-version 3\n")
    for gff in gffs:
        attribute_string = ";".join(
            ["=".join(items) for items in gff.attributes.items()])
        out_cds.write("\t".join([str(field) for field in [
                        gff.seq_id, gff.source, gff.feature, gff.start,
                        gff.end, gff.score, gff.strand, gff.phase,
                        attribute_string]]) + "\n")
    out = open(stat_file, "w")
    out.write("All genomes:\n")
    print_stat("cds", num_all, out)
    print_stat("tRNA", num_all, out)
    print_stat("rRNA", num_all, out)
    if len(num_strain) > 1:
        for strain in num_strain.keys():
            out.write(strain + ":\n")
            print_stat("cds", num_strain[strain], out)
            print_stat("tRNA", num_strain[strain], out)
            print_stat("rRNA", num_strain[strain], out)
    out_cds.close()
    out.close()


def validate_gff(tss_file, gff_file, stat_file, out_cds_file, utr_length,
                 program):
    num_all = {"all_cds": 0, "all_tRNA": 0, "all_rRNA": 0,
               "cds": 0, "tRNA": 0, "rRNA": 0}
    num_strain = {}
    pre_seq_id = ""
    gffs, tsss = read_gff(gff_file, tss_file)
    for gff in gffs:
        if gff.seq_id != pre_seq_id:
            num_strain[gff.seq_id] = {"all_cds": 0, "all_tRNA": 0,
                                      "all_rRNA": 0, "cds": 0,
                                      "tRNA": 0, "rRNA": 0}
            pre_seq_id = gff.seq_id
        if gff.feature == "CDS":
            num_all["all_cds"] += 1
            num_strain[gff.seq_id]["all_cds"] += 1
        elif gff.feature == "tRNA":
            num_all["all_tRNA"] += 1
            num_strain[gff.seq_id]["all_tRNA"] += 1
        elif gff.feature == "rRNA":
            num_all["all_rRNA"] += 1
            num_strain[gff.seq_id]["all_rRNA"] += 1
        compare_tss(tsss, gff, utr_length, num_all, num_strain, program)
    print_file(gffs, out_cds_file, stat_file, num_all, num_strain)



import os
import sys
import shutil
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.get_Rfam_ribo import rbs_from_rfam
from annogesiclib.extract_RBS import extract_potential_rbs
from annogesiclib.recompute_RBS import regenerate_seq, reextract_rbs
from annogesiclib.ribo_gff import stat_and_covert2gff
from annogesiclib.modify_rbs_table import modify_table
from annogesiclib.map_ribos import mapping_ribos
from annogesiclib.rbs_overlap import rbs_overlap


class Ribos(object):
    '''detection of riboswitch and RNA thermometer'''

    def __init__(self, args_ribo):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.gff_parser = Gff3Parser()
        self.gff_path = os.path.join(args_ribo.gffs, "tmp")
        self.tss_path = os.path.join(args_ribo.tsss, "tmp")
        self.tran_path = os.path.join(args_ribo.trans, "tmp")
        self.fasta_path = os.path.join(args_ribo.fastas, "tmp")
        if (args_ribo.program == "both") or (
                args_ribo.program == "riboswitch"):
            (self.ribos_stat_folder, self.ribos_gff_outfolder,
             self.ribos_table_folder, self.ribos_scan_folder,
             self.ribos_tmp_files, self.ribos_rfam,
             self.ribos_suffixs) = self._create_out_folders(
                args_ribo.ribos_out_folder, "riboswitch",
                args_ribo.database)
        if (args_ribo.program == "both") or (
                args_ribo.program == "thermometer"):
            (self.thermo_stat_folder, self.thermo_gff_outfolder,
             self.thermo_table_folder, self.thermo_scan_folder,
             self.thermo_tmp_files, self.thermo_rfam,
             self.thermo_suffixs) = self._create_out_folders(
                args_ribo.thermo_out_folder, "RNA_thermometer",
                args_ribo.database)

    def _create_out_folders(self, out_folder, feature, database):
        stat_folder = os.path.join(out_folder, "statistics")
        gff_outfolder = os.path.join(out_folder, "gffs")
        table_folder = os.path.join(out_folder, "tables")
        scan_folder = os.path.join(out_folder, "scan_Rfam_results")
        tmp_files = {"fasta": os.path.join(
                              out_folder, "tmp_fasta"),
                     "scan": os.path.join(
                              out_folder, "tmp_scan"),
                     "table": os.path.join(
                              out_folder, "tmp_table")}
        rfam = os.path.join(database, "Rfam_" + feature + ".cm")
        suffixs = {"csv": feature + ".csv",
                   "txt": feature + "_prescan.txt",
                   "re_txt": feature + "_scan.txt",
                   "re_csv": feature + "_scan.csv"}
        return (stat_folder, gff_outfolder, table_folder, scan_folder,
                tmp_files, rfam, suffixs)

    def _run_cmscan(self, args_ribo, seq, type_, prefix, tmp_files,
                    suffixs, rfam):
        scan_file = os.path.join(tmp_files["scan"],
                                 "_".join([prefix, suffixs[type_]]))
        scan = open(scan_file, "w")
        call([args_ribo.cmscan_path, "--incE",
              str(args_ribo.e_value), "--acc", rfam, seq], stdout=scan)
        scan.close()
        return scan_file

    def _scan_extract_rfam(self, prefixs, args_ribo, tmp_files, suffixs,
                           feature, rfam):
        '''extract the seq of candidates and scanning the candidates'''
        for gff in os.listdir(self.gff_path):
            if gff.endswith(".gff"):
                prefix = gff.replace(".gff", "")
                first_seq = os.path.join(tmp_files["fasta"],
                                         prefix + ".fa")
                prefixs.append(prefix)
                print("Extracting sequences of candidates for {0}".format(
                      prefix))
                extract_potential_rbs(
                      os.path.join(self.fasta_path, prefix + ".fa"),
                      os.path.join(self.gff_path, gff),
                      os.path.join(self.tss_path, prefix + "_TSS.gff"),
                      os.path.join(self.tran_path, prefix + "_transcript.gff"),
                      first_seq, args_ribo, feature)
                print("Pre-scanning of {0}".format(prefix))
                first_scan_file = self._run_cmscan(
                        args_ribo, first_seq, "txt", prefix, tmp_files,
                        suffixs, rfam)
                sec_seq = os.path.join(tmp_files["fasta"],
                                       "_".join([prefix, "regenerate.fa"]))
                first_table = os.path.join(
                        tmp_files["table"],
                        "_".join([prefix, suffixs["csv"]]))
                regenerate_seq(first_scan_file, first_seq,
                               first_table, sec_seq)
                print("Scanning of {0}".format(prefix))
                sec_scan_file = self._run_cmscan(
                        args_ribo, sec_seq, "re_txt", prefix, tmp_files,
                        suffixs, rfam)
                sec_table = os.path.join(
                        tmp_files["table"],
                        "_".join([prefix, suffixs["re_csv"]]))
                reextract_rbs(sec_scan_file, first_table, sec_table)
                shutil.move(sec_table, first_table)
                modify_table(first_table, args_ribo.output_all)
        return prefixs

    def _merge_results(self, args_ribo, scan_folder, suffixs, tmp_files,
                       table_folder, stat_folder, feature_id, gff_outfolder,
                       feature):
        '''merge the results from the results of two searching'''
        for gff in os.listdir(args_ribo.gffs):
            if gff.endswith(".gff"):
                prefix = gff.replace(".gff", "")
                print("Merging results of {0}".format(prefix))
                pre_strain = ""
                self.helper.check_make_folder(os.path.join(
                                              scan_folder, prefix))
                fh = open(os.path.join(args_ribo.gffs, gff))
                for entry in self.gff_parser.entries(fh):
                    if entry.seq_id != pre_strain:
                        if len(pre_strain) == 0:
                            shutil.copyfile(os.path.join(
                                tmp_files["table"],
                                "_".join([entry.seq_id, suffixs["csv"]])),
                                os.path.join(
                                    table_folder,
                                    "_".join([prefix, suffixs["csv"]])))
                        else:
                            self.helper.merge_file(os.path.join(
                                tmp_files["table"],
                                "_".join([entry.seq_id, suffixs["csv"]])),
                                os.path.join(
                                    table_folder,
                                    "_".join([prefix, suffixs["csv"]])))
                        shutil.copy(os.path.join(
                            tmp_files["scan"],
                            "_".join([entry.seq_id, suffixs["txt"]])),
                            os.path.join(scan_folder, prefix))
                        shutil.copy(os.path.join(
                            tmp_files["scan"],
                            "_".join([entry.seq_id, suffixs["re_txt"]])),
                            os.path.join(scan_folder, prefix))
                        pre_strain = entry.seq_id
                out_stat = os.path.join(
                        stat_folder,
                        "_".join(["stat", prefix, feature + ".txt"]))
                print("Computing statistics of {0}".format(prefix))
                stat_and_covert2gff(os.path.join(
                    table_folder, "_".join([prefix, suffixs["csv"]])),
                    feature_id, os.path.join(gff_outfolder,
                        "_".join([prefix, feature + ".gff"])),
                    args_ribo.fuzzy, out_stat, feature)
                fh.close()

    def _remove_tmp(self, args_ribo):
        self.helper.remove_tmp_dir(args_ribo.gffs)
        self.helper.remove_tmp_dir(args_ribo.fastas)
        self.helper.remove_tmp_dir(args_ribo.trans)
        self.helper.remove_tmp_dir(args_ribo.tsss)

    def _remove_overlap(self, gff_path, tmp_files, suffixs):
        for gff in os.listdir(gff_path):
            if gff.endswith(".gff"):
                rbs_overlap(
                    os.path.join(os.path.join(
                        tmp_files["table"],
                        "_".join([gff.replace(".gff", ""),
                                  suffixs["csv"]]))),
                    os.path.join(gff_path, gff))

    def _core_prediction(self, args_ribo, feature_id, rfam, tmp_files,
                         table_folder, feature, scan_folder, suffixs,
                         stat_folder, gff_outfolder, out_folder):
        '''main part of detection'''
        rbs_from_rfam(feature_id, args_ribo.rfam, rfam)
        print("Compressing Rfam of " + feature)
        call([args_ribo.cmpress_path, "-F", rfam])
        prefixs = []
        self.helper.check_make_folder(tmp_files["fasta"])
        self.helper.check_make_folder(tmp_files["scan"])
        self.helper.check_make_folder(tmp_files["table"])
        prefixs = self._scan_extract_rfam(
                prefixs, args_ribo, tmp_files, suffixs, feature, rfam)
        self._remove_overlap(self.gff_path, tmp_files, suffixs)
        self._merge_results(args_ribo, scan_folder, suffixs, tmp_files,
                            table_folder, stat_folder, feature_id,
                            gff_outfolder, feature)
        mapping_ribos(table_folder, feature_id, feature)
        self.helper.remove_all_content(out_folder, "tmp", "dir")

    def run_ribos(self, args_ribo):
        if args_ribo.fuzzy_rbs > 6:
            print("Error: --fuzzy_rbs should be equal or less than 6!!")
            sys.exit()
        self.multiparser.parser_gff(args_ribo.gffs, None)
        self.multiparser.parser_fasta(args_ribo.fastas)
        self.multiparser.parser_gff(args_ribo.trans, "transcript")
        self.multiparser.parser_gff(args_ribo.tsss, "TSS")
        for gff in os.listdir(args_ribo.gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(
                                                 args_ribo.gffs, gff))
        if (args_ribo.program.lower() == "both") or (
                args_ribo.program.lower() == "riboswitch"):
            print("Detecting riboswtiches now")
            self._core_prediction(
                    args_ribo, args_ribo.ribos_id, self.ribos_rfam,
                    self.ribos_tmp_files, self.ribos_table_folder,
                    "riboswitch", self.ribos_scan_folder, self.ribos_suffixs,
                    self.ribos_stat_folder, self.ribos_gff_outfolder,
                    args_ribo.ribos_out_folder)
        if (args_ribo.program.lower() == "both") or (
                args_ribo.program.lower() == "thermometer"):
            print("Detecting RNA thermometers now")
            self._core_prediction(
                    args_ribo, args_ribo.thermo_id, self.thermo_rfam,
                    self.thermo_tmp_files, self.thermo_table_folder,
                    "RNA_thermometer", self.thermo_scan_folder,
                    self.thermo_suffixs, self.thermo_stat_folder,
                    self.thermo_gff_outfolder, args_ribo.thermo_out_folder)
        self._remove_tmp(args_ribo)



import os
import sys
from annogesiclib.projectcreator import ProjectCreator
from annogesiclib.paths import Paths
from annogesiclib.get_input import get_file
from annogesiclib.converter import Converter
from annogesiclib.get_target_fasta import TargetFasta
from annogesiclib.ratt import RATT
from annogesiclib.tsspredator import TSSpredator
from annogesiclib.optimize import optimize_tss
from annogesiclib.color_png import ColorPNG
from annogesiclib.terminator import Terminator
from annogesiclib.transcript import TranscriptDetection
from annogesiclib.utr import UTRDetection
from annogesiclib.srna import sRNADetection
from annogesiclib.sorf import sORFDetection
from annogesiclib.meme import MEME
from annogesiclib.operon import OperonDetection
from annogesiclib.circrna import CircRNADetection
from annogesiclib.goterm import GoTermFinding
from annogesiclib.srna_target import sRNATargetPrediction
from annogesiclib.snp import SNPCalling
from annogesiclib.ppi import PPINetwork
from annogesiclib.sublocal import SubLocal
from annogesiclib.ribos import Ribos
from annogesiclib.crispr import Crispr
from annogesiclib.merge_feature import run_merge
from annogesiclib.screen import Screen
from annogesiclib.args_container import ArgsContainer
from annogesiclib.helper import Helper
project_creator = ProjectCreator()


class Controller(object):

    """Manage the actions of the subcommands.

    The Controller take care of providing the argumentes like path
    names and the parallel processing of tasks.

    """
    def __init__(self, args):
        """Create an instance."""
        self._args = args
        self._paths = Paths(args.project_path)
        self.args_container = ArgsContainer()
        self.helper = Helper()

    def check_folder(self, folders, flags):
        '''Check the emtpy or wrong assigned folder'''
        for folder, flag in zip(folders, flags):
            if folder is None:
                print("Error: {0} is wrong. Please check it!".format(flag))
                sys.exit()
            else:
                if os.path.exists(folder):
                    if len(os.listdir(folder)) == 0:
                        print("Error: {0} is a empty folder!".format(flag))
                        sys.exit()
                else:
                    print("Error: {0} is wrong. Please check it!".format(
                          flag))
                    sys.exit()

    def check_multi_files(self, input_files, flags):
        if input_files is not None:
            for files, flag in zip(input_files, flags):
                if files is not None:
                    for file_ in files:
                        if not os.path.exists(file_):
                            print("Error: Some files in {0} do "
                                  "not exist!".format(flag))
                            sys.exit()

    def check_parameter(self, paras, names):
        '''Check the parameter is assigned correct or not'''
        for i in range(len(paras)):
            if paras[i] is None:
                print("Error: {0} is wrong. "
                      "Please check it!".format(names[i]))
                sys.exit()

    def check_no_require_folder(self, folders):
        '''Check the folders which are not necessary.
        It should not be assigned a empty or wrong folder'''
        for folder in folders:
            if folder is not None:
                if os.path.exists(folder):
                    if len(os.listdir(folder)) == 0:
                        print("Error: There is a empty folder. "
                              "Please check it!")
                        sys.exit()
                else:
                    print("Error: There is a wrong folder. "
                          "Please check it!")
                    sys.exit()

    def check_execute_file(self, exe):
        detect = False
        if os.path.exists(exe):
            detect = True
            full_exe = os.path.realpath(exe)
        for folder in os.environ["PATH"].split(":"):
            if os.path.exists(os.path.join(folder, exe)):
                detect = True
                full_exe = exe
        if not detect:
            if os.path.exists(os.path.realpath(exe)):
                full_exe = os.path.realpath(exe)
            else:
                print("Error: {0} can't be found!".format(exe))
                print("Please assign the correct path!")
                sys.exit()
        return full_exe

    def check_file(self, files, names, require):
        '''Check the path of file'''
        for i in range(len(files)):
            if require:
                if files[i] is None:
                    print("Error: {0} is wrong. "
                          "Please check it!".format(names[i]))
                    sys.exit()
                else:
                    if not os.path.isfile(files[i]):
                        print("Error: There is a wrong path of {0}. "
                              "Please check it!".format(names[i]))
                        sys.exit()
            else:
                if files[i] is not None:
                    if not os.path.isfile(files[i]):
                        print("Error: There is a wrong path of {0}. "
                              "Please check it!".format(names[i]))
                        sys.exit()

    def create_project(self, version):
        """Create a new project."""
        project_creator.create_root_folder(self._args.project_path)
        project_creator.create_subfolders(self._paths.required_folders("root"))
        project_creator.create_version_file(
            self._paths.version_path, version)
        sys.stdout.write("Created folder \"%s\" and required subfolders.\n" % (
            self._args.project_path))

    def get_input(self):
        """Download required files from website."""
        print("Running get input files")
        if self._args.ftp_path is None:
            print("Error: Please assign the path for downloading the data!")
            sys.exit()
            annotation_folder = self._paths.ref_annotation_folder
            fasta_folder = self._paths.ref_fasta_folder
        self.helper.check_make_folder(self._paths.ref_annotation_folder)
        self.helper.check_make_folder(self._paths.ref_fasta_folder)
        if self._args.ref_gff is True:
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "gff")
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "_genomic.gff.gz")
        if self._args.ref_fasta is True:
            get_file(self._args.ftp_path, self._paths.ref_fasta_folder,
                     "fna")
            get_file(self._args.ftp_path, self._paths.ref_fasta_folder,
                     "_genomic.fna.gz")
        if self._args.ref_gbk is True:
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "gbk")
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "gbff")
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "_genomic.gbff.gz")
        if self._args.ref_ptt is True:
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "ptt")
        if self._args.ref_rnt is True:
            get_file(self._args.ftp_path, self._paths.ref_annotation_folder,
                     "rnt")
        if self._args.convert_embl is True:
            annotation_files = os.listdir(self._paths.ref_annotation_folder)
            if len(annotation_files) == 0:
                sys.stdout.write("No gff files!!\n")
            else:
                Converter().convert_gbk2embl(self._paths.ref_annotation_folder)

    def get_target_fasta(self):
        """Get target fasta"""
        print("Running update genome fasta")
        self.check_multi_files([self._args.related_fasta_files],
                               ["--related_fasta_files"])
        self.check_file([self._args.mutation_table], "--mutation_table", True)
        project_creator.create_subfolders(
            self._paths.required_folders("get_target_fasta"))
        target = TargetFasta(self._paths.tar_fasta_folder,
                             self._args.related_fasta_files)
        target.get_target_fasta(
                self._args.mutation_table, self._paths.tar_fasta_folder,
                self._args.related_fasta_files, self._args.combine_to_one_fasta,
                self._paths.target_base_folder)

    def ratt(self):
        """Run RATT to transfer annotation file from reference to target."""
        print("Running annotation transfer")
        if (self._args.transfer_type != "Strain") and (
                self._args.transfer_type != "Assembly") and (
                self._args.transfer_type != "Species") and (
                self._args.transfer_type != "Assembly.Repetitive") and (
                self._args.transfer_type != "Strain.Repetitive") and (
                self._args.transfer_type != "Species.Repetitive") and (
                self._args.transfer_type != "Multiple") and (
                self._args.transfer_type != "Free"):
            print("Error: please assign correct --transfer_type!")
            sys.exit()
        if (self._args.related_embl_files is None) and (
                self._args.related_gbk_files is None):
            print("Error: please assign proper embl or genbank files")
            sys.exit()
        elif (self._args.related_embl_files is not None) and (
                self._args.related_gbk_files is not None):
            print("Error: please choose embl as input or genbank as input")
            sys.exit()
        self._args.ratt_path = self.check_execute_file(self._args.ratt_path)
        self.check_multi_files(
                [self._args.updated_fasta_files, self._args.related_fasta_files],
                ["--updated_fasta_files", "--closed_fasta_files"])
        self.check_parameter([self._args.element, self._args.compare_pair],
                             ["--element", "--compare_pair"])
        project_creator.create_subfolders(
            self._paths.required_folders("annotation_transfer"))
        args_ratt = self.args_container.container_ratt(
            self._args.ratt_path, self._args.element, self._args.transfer_type,
            self._args.related_embl_files, self._args.related_gbk_files,
            self._args.updated_fasta_files, self._args.related_fasta_files,
            self._paths.ratt_folder, self._args.convert_to_gff_rnt_ptt,
            self._paths.tar_annotation_folder, self._args.compare_pair)
        ratt = RATT(args_ratt)
        ratt.annotation_transfer(args_ratt)

    def tsspredator(self):
        """Run TSSpredator for predicting TSS candidates."""
        self.check_multi_files(
                [self._args.fasta_files, self._args.annotation_files,
                 self._args.compare_overlap_gff, self._args.manual_files,
                 self._args.compare_transcript_files],
                ["--fasta_files", "--annotation_files", "--compare_overlap_gff",
                 "--manual_files","--compare_transcript_files"])
        self.check_parameter([self._args.tex_notex_libs, self._args.condition_names],
                             ["--tex_notex_libs", "--condition_names"])
        self._args.tsspredator_path = self.check_execute_file(
                self._args.tsspredator_path)
        if self._args.program.lower() == "tss":
            print("Running TSS prediction")
            project_creator.create_subfolders(
                self._paths.required_folders("TSS"))
            out_folder = self._paths.tsspredator_folder
        elif self._args.program.lower() == "ps":
            print("Running processing site prediction")
            out_folder = self._paths.processing_site_folder
            project_creator.create_subfolders(
                self._paths.required_folders("processing"))
        else:
            print("Error: No such program!")
            sys.exit()
        args_tss = self.args_container.container_tsspredator(
            self._args.tsspredator_path, self._args.program,
            self._args.fasta_files, self._args.annotation_files,
            self._args.tex_notex_libs, self._args.condition_names,
            self._args.height, self._args.height_reduction,
            self._args.factor, self._args.factor_reduction,
            self._args.base_height, self._args.enrichment_factor,
            self._args.processing_factor, self._args.replicate_tex,
            out_folder, self._args.validate_gene,
            self._args.manual_files, self._args.curated_sequence_length,
            self._args.compare_transcript_files, self._args.tolerance,
            self._args.utr_length, self._args.cluster,
            self._args.re_check_orphan,
            self._args.remove_overlap_feature, self._args.compare_overlap_gff,
            self._args.remove_low_expression)
        tsspredator = TSSpredator(args_tss)
        tsspredator.run_tsspredator(args_tss)

    def optimize(self):
        """opimize TSSpredator"""
        self.check_multi_files(
                [self._args.fasta_files, self._args.annotation_files,
                 self._args.manual_files],
                ["--fasta_files", "--annotation_files", "--manual_files"])
        self._args.tsspredator_path = self.check_execute_file(
                self._args.tsspredator_path)
        self.check_parameter([self._args.tex_notex_libs,
                              self._args.condition_names],
                             ["--tex_notex_lib",
                              "--condition_names"])
        if self._args.program.lower() == "tss":
            print("Running optimization of TSS prediction")
            project_creator.create_subfolders(
                self._paths.required_folders("TSS"))
            out_folder = self._paths.tsspredator_folder
        elif self._args.program.lower() == "ps":
            print("Running optimization of processing site prediction")
            out_folder = self._paths.processing_site_folder
            project_creator.create_subfolders(
                self._paths.required_folders("processing"))
        else:
            print("Error: No such program!")
            sys.exit()
        args_ops = self.args_container.container_optimize(
            self._args.tsspredator_path, self._args.fasta_files,
            self._args.annotation_files,
            self._args.manual_files, out_folder, self._args.max_height,
            self._args.max_height_reduction, self._args.max_factor,
            self._args.max_factor_reduction, self._args.max_base_height,
            self._args.max_enrichment_factor, self._args.max_processing_factor,
            self._args.utr_length, self._args.tex_notex_libs,
            self._args.condition_names, self._args.cluster,
            self._args.curated_sequence_length, self._args.parallels,
            self._args.program, self._args.replicate_tex,
            self._args.steps)
        optimize_tss(args_ops)

    def color(self):
        """color the screenshots"""
        print("Running png files coloring")
        self.check_parameter([self._args.track_number], ["--track_numer"])
        self.check_folder([self._args.screenshot_folder], ["--screenshot_folder"])
        self._args.imagemagick_covert_path = self.check_execute_file(
                self._args.imagemagick_covert_path)
        color = ColorPNG()
        color.generate_color_png(
                self._args.track_number, self._args.screenshot_folder,
                self._args.imagemagick_covert_path)

    def terminator(self):
        """Run TransTermHP and Gene converaged for detecting terminators"""
        print("Running terminator prediction")
        if self._args.transterm_path is None:
            print("Please assign the path of transterm in TransTermHP.")
        self.check_multi_files(
                [self._args.fasta_files, self._args.annotation_files,
                 self._args.transcript_files, self._args.srna_files],
                ["--fasta_files", "--annotation_files",
                 "--transcript_files", "--srna_files"])
        for prop in ("transterm_path", "expterm_path", "rnafold_path"):
            setattr(self._args, prop,
                    self.check_execute_file(getattr(self._args, prop)))
        project_creator.create_subfolders(
            self._paths.required_folders("terminator"))
        args_term = self.args_container.container_terminator(
            self._args.transterm_path, self._args.expterm_path,
            self._args.rnafold_path,
            self._paths.transterm_folder, self._args.fasta_files,
            self._args.annotation_files, self._args.transcript_files,
            self._args.srna_files, self._args.decrease,
            self._args.highest_coverage, self._args.tolerance_detect_coverage,
            self._args.tolerance_within_transcript,
            self._args.tolerance_downstream_transcript,
            self._args.tolerance_within_gene,
            self._args.tolerance_downstream_gene, self._paths.transtermhp_folder,
            self._args.tex_notex_libs, self._args.frag_libs,
            self._args.tex_notex, self._args.replicate_tex,
            self._args.replicate_frag, self._args.table_best,
            self._args.min_loop_length, self._args.max_loop_length,
            self._args.min_stem_length, self._args.max_stem_length,
            self._args.min_u_tail_length, self._args.miss_rate,
            self._args.range_u_tail, self._args.keep_multi_term,
            self._args.window_size, self._args.window_shift)
        terminator = Terminator(args_term)
        terminator.run_terminator(args_term)

    def transcript(self):
        """Run Transcript detection"""
        print("Running transcript detection")
        self.check_multi_files(
                [self._args.annotation_files, self._args.tss_files,
                 self._args.terminator_files],
                ["--annotation_files", "--tss_files", "--terminator_files"])
        project_creator.create_subfolders(
            self._paths.required_folders("transcript"))
        args_tran = self.args_container.container_transcript(
            self._args.tex_notex, self._args.modify_transcript,
            self._args.length, self._args.annotation_files,
            self._args.height, self._args.width,
            self._args.tolerance, self._args.tolerance_coverage,
            self._args.replicate_tex, self._args.replicate_frag,
            self._paths.transcript_output_folder,
            self._args.tss_files, self._args.tss_tolerance,
            self._args.tex_notex_libs, self._args.frag_libs,
            self._args.compare_feature_genome, self._args.table_best,
            self._args.terminator_files, self._args.terminator_tolerance,
            self._args.max_length_distribution)
        transcript = TranscriptDetection(args_tran)
        transcript.run_transcript(args_tran)

    def utr_detection(self):
        """Run UTR detection."""
        print("Running UTR detection")
        self.check_multi_files(
            [self._args.annotation_files, self._args.terminator_files,
             self._args.transcript_files, self._args.tss_files],
            ["--annotation_files", "--terminator_files",
             "--transcript_files", "--tss_files"])
        project_creator.create_subfolders(self._paths.required_folders("utr"))
        args_utr = self.args_container.container_utr(
                self._args.tss_files, self._args.annotation_files,
                self._args.transcript_files, self._args.terminator_files,
                self._args.terminator_tolerance, self._paths.utr_folder,
                self._args.tss_source, self._args.base_5utr,
                self._args.utr_length, self._args.base_3utr,
                self._args.tolerance_3utr, self._args.tolerance_5utr)
        utr = UTRDetection(args_utr)
        utr.run_utr_detection(args_utr)

    def _check_filter_input(self, files, info, filters):
        if files is None:
            print("Error: The {0} has to be provided "
                  "if \"{1}\" in --filter_info!".format(info, filters))
            sys.exit()

    def _check_database(self, database, flag, info):
        wrong = False
        if database is None:
            wrong = True
        elif not os.path.isfile(database):
            if (os.path.isfile(database + ".fa")) or (
                    os.path.isfile(database + ".fna")) or (
                    os.path.isfile(database + ".fasta")):
                wrong = False
            else:
                wrong = True
        if wrong:
            print("Error: {0} is required if {1} is in --filter_info. "
                  "But the assignment of {0} is empty or wrong. "
                  "Please check the {0} or remove {1} from "
                  "--filter_info!".format(flag, info))
            sys.exit()

    def srna_detection(self):
        """sRNA_detection."""
        print("Running sRNA prediction")
        self.check_multi_files(
                [self._args.annotation_files, self._args.transcript_files,
                 self._args.fasta_files, self._args.sorf_files,
                 self._args.terminator_files, self._args.promoter_tables,
                 self._args.processing_site_files],
                ["--annotation_files", "--transcript_files",
                 "--fasta_files", "--sorf_files", "--terminator_files",
                 "--promoter_tables", "--processing_site_files"])
        for info in self._args.filter_info:
            if "sec_str" == info:
                if not self._args.compute_sec_structures:
                    print("Error: --compute_sec_structures is not switch on, "
                          "but sec_str is still in --filter_info.")
                    sys.exit()
                self._check_filter_input(
                        self._args.fasta_files, "fasta file", "sec_str")
                for prop in ("rnafold_path", "relplot_path",
                             "mountain_path"):
                    setattr(self._args, prop,
                            self.check_execute_file(getattr(self._args, prop)))
            elif ("blast_nr" == info) or (
                    "blast_srna"== info):
                for prop in ("blastn_path", "blastx_path", "makeblastdb_path"):
                    setattr(self._args, prop,
                            self.check_execute_file(getattr(self._args, prop)))
                if ("blast_nr" == info):
                    self._check_database(self._args.nr_database_path,
                                         "--nr_database_path", "blast_nr")
                if ("blast_srna" == info):
                    self._check_database(self._args.srna_database_path,
                                         "--srna_database_path", "blast_srna")
            elif "sorf" == info:
                self._check_filter_input(
                        self._args.sorf_files, "sORF", "sorf")
            elif "term" == info:
                self._check_filter_input(self._args.terminator_files,
                                         "terminator", "term")
            elif "promoter" == info:
                self._check_filter_input(self._args.promoter_tables,
                                         "Promoter", "promoter")
            elif "tss" == info:
                self._check_filter_input(self._args.tss_files,
                                         "TSS", "tss")
            else:
                if "none" != info.lower():
                    print("Error: Please check the --filter_info, "
                          "invalid value was assigned!")
                    sys.exit()
        if self._args.utr_derived_srna:
            if self._args.tss_files is None:
                print("Error: The TSS has to be provided "
                      "if you want to compute UTR-derived sRNA!")
                sys.exit()
        project_creator.create_subfolders(self._paths.required_folders("srna"))
        args_srna = self.args_container.container_srna(
                self._args.rnafold_path, self._args.relplot_path,
                self._args.mountain_path, self._args.blastn_path,
                self._args.blastx_path, self._args.makeblastdb_path,
                self._paths.srna_folder, self._args.utr_derived_srna,
                self._args.annotation_files, self._args.tss_files,
                self._args.transcript_files,
                self._args.tss_intergenic_antisense_tolerance,
                self._args.tss_5utr_tolerance, self._args.tss_3utr_tolerance,
                self._args.tss_intercds_tolerance, self._args.filter_info,
                self._args.processing_site_files, self._args.fasta_files,
                self._args.mountain_plot, self._args.nr_format,
                self._args.srna_format, self._args.srna_database_path,
                self._args.nr_database_path, self._args.cutoff_energy,
                self._args.parallel_blast,
                self._args.min_intergenic_tex_coverage,
                self._args.min_intergenic_notex_coverage,
                self._args.min_intergenic_fragmented_coverage,
                self._args.min_complete_5utr_transcript_coverage,
                self._args.min_antisense_tex_coverage,
                self._args.min_antisense_notex_coverage,
                self._args.min_antisense_fragmented_coverage,
                self._args.min_utr_tex_coverage,
                self._args.min_utr_notex_coverage,
                self._args.min_utr_fragmented_coverage,
                self._args.max_length, self._args.min_length,
                self._args.tex_notex_libs, self._args.frag_libs,
                self._args.replicate_tex, self._args.replicate_frag,
                self._args.tex_notex, self._args.blast_e_nr,
                self._args.blast_e_srna, self._args.detect_srna_in_cds,
                self._args.table_best, self._args.decrease_intergenic_antisense,
                self._args.decrease_utr, self._args.tolerance_intergenic_antisense,
                self._args.tolerance_utr, self._args.cutoff_nr_hit,
                self._args.sorf_files, self._args.overlap_percent_cds,
                self._args.terminator_files,
                self._args.terminator_tolerance_in_srna,
                self._args.terminator_tolerance_out_srna,
                self._args.ignore_hypothetical_protein, self._args.tss_source,
                self._args.min_all_utr_coverage, self._args.promoter_tables,
                self._args.ranking_time_promoter, self._args.promoter_names,
                self._args.compute_sec_structures)
        srna = sRNADetection(args_srna)
        srna.run_srna_detection(args_srna)

    def sorf_detection(self):
        """sORF_detection."""
        print("Running sORF prediction")
        self.check_multi_files(
                [self._args.transcript_files, self._args.annotation_files,
                 self._args.fasta_files, self._args.srna_files,
                 self._args.tss_files],
                ["--transcript_files", "--annotation_files",
                 "--fasta_files", "--srna_files", "--tss_files"])
        project_creator.create_subfolders(
            self._paths.required_folders("sorf"))
        args_sorf = self.args_container.container_sorf(
            self._paths.sorf_folder, self._args.utr_derived_sorf,
            self._args.transcript_files,
            self._args.annotation_files,
            self._args.tss_files, self._args.utr_length,
            self._args.min_length, self._args.max_length,
            self._args.cutoff_intergenic_coverage,
            self._args.cutoff_antisense_coverage,
            self._args.cutoff_5utr_coverage,
            self._args.cutoff_3utr_coverage,
            self._args.cutoff_intercds_coverage,
            self._args.fasta_files, self._args.tex_notex_libs,
            self._args.frag_libs, self._args.tex_notex,
            self._args.replicate_tex, self._args.replicate_frag,
            self._args.table_best, self._args.srna_files,
            self._args.start_codon, self._args.stop_codon,
            self._args.cutoff_base_coverage, self._args.tolerance_rbs,
            self._args.rbs_not_after_tss, self._args.print_all_combination,
            self._args.best_no_srna, self._args.best_no_tss,
            self._args.ignore_hypothetical_protein,
            self._args.min_rbs_distance, self._args.max_rbs_distance)
        sorf = sORFDetection(args_sorf)
        sorf.run_sorf_detection(args_sorf)

    def meme(self):
        """promoter detectopn"""
        print("Running promoter detection")
        self.check_multi_files(
                [self._args.tss_files, self._args.fasta_files],
                ["--tss_files", "--fasta_files"])
        if not self._args.tss_source:
            self.check_multi_files([self._args.annotation_files],
                                   ["--annotation_files"])
        if (self._args.program == "both") or (
                self._args.program == "meme"):
            self._args.meme_path = self.check_execute_file(self._args.meme_path)
        elif (self._args.program == "both") or (
                self._args.program == "glam2"):
            self._args.glam2_path = self.check_execute_file(self._args.glam2_path)
        project_creator.create_subfolders(
            self._paths.required_folders("promoter"))
        args_pro = self.args_container.container_promoter(
            self._args.meme_path, self._args.glam2_path,
            self._paths.promoter_output_folder, self._args.tex_libs,
            self._args.tss_files, self._args.fasta_files,
            self._args.num_motifs, self._args.nt_before_tss,
            self._args.motif_width, self._args.tss_source,
            self._args.annotation_files, self._args.end_run,
            self._args.combine_all, self._args.e_value,
            self._args.parallels, self._args.program)
        meme = MEME(args_pro)
        meme.run_meme(args_pro)

    def operon(self):
        """operon detection"""
        print("Running operon detection")
        self.check_multi_files(
                [self._args.tss_files, self._args.annotation_files,
                 self._args.transcript_files, self._args.utr5_files,
                 self._args.utr3_files, self._args.terminator_files],
                ["--tss_files", "--annotation_files",
                 "--transcript_files", "--utr5_files",
                 "--utr3_files", "--terminator_files"])
        project_creator.create_subfolders(
            self._paths.required_folders("operon"))
        args_op = self.args_container.container_operon(
            self._args.tss_files, self._args.annotation_files,
            self._args.transcript_files, self._args.utr5_files,
            self._args.utr3_files, self._args.terminator_files,
            self._args.tss_tolerance, self._args.terminator_tolerance,
            self._args.min_length, self._paths.operon_output_folder,
            self._paths.operon_statistics_folder)
        operon = OperonDetection(args_op)
        operon.run_operon(args_op)

    def circrna(self):
        """circRNA detection"""
        print("Running circular RNA prediction")
        if self._args.read_files:
            self._args.segemehl_path = self.check_execute_file(
                    self._args.segemehl_path)
        for prop in ("testrealign_path", "samtools_path"):
            setattr(self._args, prop,
                    self.check_execute_file(getattr(self._args, prop)))
        self.check_multi_files(
                [self._args.fasta_files, self._args.annotation_files],
                ["--fasta_files", "--annotation_files"])
        project_creator.create_subfolders(
            self._paths.required_folders("circrna"))
        args_circ = self.args_container.container_circrna(
            self._args.parallels, self._args.fasta_files,
            self._args.annotation_files, self._args.bam_files,
            self._args.read_files, self._paths.circrna_stat_folder,
            self._args.support_reads, self._args.segemehl_path,
            self._args.testrealign_path, self._args.samtools_path,
            self._args.start_ratio, self._args.end_ratio,
            self._args.ignore_hypothetical_protein,
            self._paths.circrna_output_folder)
        circ = CircRNADetection(args_circ)
        circ.run_circrna(args_circ)

    def goterm(self):
        """Go term discovery"""
        print("Running GO term mapping")
        self.check_multi_files(
                [self._args.annotation_files, self._args.transcript_files],
                ["--annotation_files", "--transcript_files"])
        self.check_file([self._args.uniprot_id, self._args.go_obo,
                         self._args.goslim_obo],
                        ["--uniprot_id", "--go.obo", "--goslim_obo"], True)
        project_creator.create_subfolders(
            self._paths.required_folders("go_term"))
        args_go = self.args_container.container_goterm(
            self._args.annotation_files,
            self._paths.goterm_output_folder, self._args.uniprot_id,
            self._args.go_obo, self._args.goslim_obo,
            self._args.transcript_files)
        goterm = GoTermFinding(args_go)
        goterm.run_go_term(args_go)

    def srna_target(self):
        """sRNA target prediction"""
        print("Running sRNA target prediction")
        self.check_multi_files(
                [self._args.fasta_files, self._args.srna_files,
                 self._args.annotation_files],
                ["--fasta_files", "--srna_files",
                 "--annotation_files"])
        if (self._args.program.lower() == "both"):
            for prop in ("rnaplfold_path", "rnaplex_path", "rnaup_path"):
                setattr(self._args, prop,
                        self.check_execute_file(getattr(self._args, prop)))
        elif self._args.program.lower() == "RNAup":
            self._args.rnaup_path = self.check_execute_file(self._args.rnaup_path)
        elif self._args.program.lower() == "RNAplex":
            for prop in ("rnaplfold_path", "rnaplex_path"):
                setattr(self._args, prop,
                        self.check_execute_file(getattr(self._args, prop)))
        project_creator.create_subfolders(
            self._paths.required_folders("srna_target"))
        args_tar = self.args_container.container_srna_target(
            self._args.rnaplfold_path, self._args.rnaplex_path,
            self._args.rnaup_path, self._args.annotation_files,
            self._args.fasta_files, self._args.srna_files,
            self._args.query_srnas, self._args.program,
            self._args.interaction_length, self._args.window_size_target,
            self._args.span_target, self._args.window_size_srna,
            self._args.span_srna,
            self._args.unstructured_region_rnaplex_target,
            self._args.unstructured_region_rnaplex_srna,
            self._args.unstructured_region_rnaup, self._args.energy_threshold,
            self._args.duplex_distance, self._args.top,
            self._paths.starget_output_folder, self._args.parallels_rnaplex,
            self._args.parallels_rnaup, self._args.continue_rnaup,
            self._args.potential_target_start, self._args.potential_target_end,
            self._args.target_feature)
        srnatarget = sRNATargetPrediction(args_tar)
        srnatarget.run_srna_target_prediction(args_tar)

    def snp(self):
        """SNP transcript detection"""
        print("Running SNP/mutations calling")
        self.check_multi_files(
                [self._args.fasta_files],
                ["--fasta_files"])
        if (self._args.bam_type != "related_genome") and (
                self._args.bam_type != "reference_genome"):
            print("Error: Please assign \"related_genome\" or"
                  " \"reference_genome\" to --bam_type!")
            sys.exit()
        if (self._args.ploidy != "haploid") and (
                self._args.ploidy != "diploid"):
            print("Error: Please assign \"haploid\" or"
                  " \"diploid\" to --chromosome_type!")
        if (self._args.caller != "c") and (
                self._args.caller != "m"):
            print("Error: Please assign \"c\" or"
                  " \"m\" to --caller!")
        for prop in ("bcftools_path", "samtools_path"):
            setattr(self._args, prop,
                    self.check_execute_file(getattr(self._args, prop)))
        project_creator.create_subfolders(self._paths.required_folders("snp"))
        args_snp = self.args_container.container_snp(
            self._args.samtools_path, self._args.bcftools_path,
            self._args.bam_type,
            self._args.program, self._args.fasta_files,
            self._args.bam_files,
            self._args.quality, self._args.read_depth_range,
            self._paths.snp_output_folder, self._args.indel_fraction,
            self._args.ploidy, self._args.rg_tag, self._args.caller,
            self._args.filter_tag_info, self._args.dp4_cutoff)
        snp = SNPCalling(args_snp)
        snp.run_snp_calling(args_snp)

    def ppi(self):
        """PPI network retrieve"""
        print("Running protein-protein interaction networks prediction")
        self.check_multi_files([self._args.annotation_files],
                               ["--annotation_files"])
        self.check_parameter([self._args.query_strains,
                              self._args.species_string],
                             ["--query_strains", "--species_string"])
        project_creator.create_subfolders(
            self._paths.required_folders("ppi_network"))
        args_ppi = self.args_container.container_ppi(
            self._args.annotation_files, self._args.query_strains,
            self._args.without_strain_pubmed, self._args.species_string,
            self._args.score, self._paths.ppi_output_folder,
            self._args.node_size, self._args.query)
        ppi = PPINetwork(self._paths.ppi_output_folder)
        ppi.retrieve_ppi_network(args_ppi)

    def sublocal(self):
        """Subcellular Localization prediction"""
        print("Running subcellular localization prediction")
        self.check_multi_files(
                [self._args.annotation_files, self._args.fasta_files,
                 self._args.transcript_files],
                ["--annotation_files", "--fasta_files",
                 "--transcript_files"])
        if (self._args.bacteria_type != "positive") and (
                self._args.bacteria_type != "negative"):
            print("Error: Please assign \"positive\" or"
                  " \"negative\" to --bacteria_type!")
            sys.exit()
        self._args.psortb_path = self.check_execute_file(self._args.psortb_path)
        project_creator.create_subfolders(
            self._paths.required_folders("subcellular_localization"))
        args_sub = self.args_container.container_sublocal(
            self._args.psortb_path, self._args.annotation_files,
            self._args.fasta_files, self._args.bacteria_type,
            self._args.difference_multi,
            self._paths.sublocal_output_folder, self._args.transcript_files)
        sublocal = SubLocal(args_sub)
        sublocal.run_sub_local(args_sub)

    def ribos(self):
        """riboswitch and RNA thermometer prediction"""
        print("Running riboswitch and RNA thermometer prediction")
        self.check_multi_files(
                [self._args.annotation_files, self._args.fasta_files,
                 self._args.tss_files, self._args.transcript_files],
                ["--annotation_files", "--fasta_files", "--tss_files",
                 "--transcript_files"])
        if (self._args.program == "both"):
            self.check_file([self._args.riboswitch_id_file, self._args.rfam_path],
                            ["--riboswitch_id_file", "--rfam_path"], True)
            self.check_file([self._args.rna_thermometer_id_file,
                             self._args.rfam_path],
                            ["--rna_thermometer_id_file", "--rfam_path"], True)
            project_creator.create_subfolders(
                    self._paths.required_folders("riboswitch"))
            project_creator.create_subfolders(
                    self._paths.required_folders("thermometer"))
            ribos_path = self._paths.ribos_output_folder
            thermo_path = self._paths.thermo_output_folder
        elif (self._args.program == "thermometer"):
            self.check_file([self._args.rna_thermometer_id_file,
                             self._args.rfam_path],
                            ["--thermometer_id_file", "--rfam_path"], True)
            project_creator.create_subfolders(
                    self._paths.required_folders("thermometer"))
            ribos_path = None
            thermo_path = self._paths.thermo_output_folder
        elif (self._args.program == "riboswitch"):
            self.check_file([self._args.riboswitch_id_file, self._args.rfam_path],
                            ["--riboswitch_id_file", "--rfam_path"], True)
            project_creator.create_subfolders(
                    self._paths.required_folders("riboswitch"))
            ribos_path = self._paths.ribos_output_folder
            thermo_path = None
        else:
            print("Error: Please assign \"thermometer\", \"riboswitch\" "
                  "or \"both\" in --program!")
            sys.exit()
        self._args.cmscan_path = self.check_execute_file(self._args.cmscan_path)
        self._args.cmpress_path = self.check_execute_file(self._args.cmpress_path)
        args_ribo = self.args_container.container_ribos(
            self._args.program, self._args.rna_thermometer_id_file,
            self._args.cmscan_path, self._args.cmpress_path,
            self._args.riboswitch_id_file,
            self._args.annotation_files, self._args.fasta_files,
            self._args.tss_files, self._args.transcript_files,
            self._args.rfam_path, ribos_path,
            thermo_path, self._args.e_value,
            self._args.output_all, self._paths.database_folder,
            self._args.tolerance, self._args.start_codon,
            self._args.min_dist_rbs, self._args.max_dist_rbs,
            self._args.tolerance_rbs, self._args.utr_length)
        ribos = Ribos(args_ribo)
        ribos.run_ribos(args_ribo)

    def crispr(self):
        """CRISPR prediction"""
        print("Running CRISPR prediction")
        self.check_multi_files(
                [self._args.fasta_files, self._args.annotation_files],
                ["--fasta_files", "--annotation_files"])
        self._args.crt_path = self.check_execute_file(self._args.crt_path)
        project_creator.create_subfolders(
            self._paths.required_folders("crispr"))
        args_cris = self.args_container.container_cris(
            self._args.fasta_files, self._args.annotation_files,
            self._args.crt_path, self._args.window_size,
            self._args.min_number_repeats, self._args.min_length_repeat,
            self._args.Max_length_repeat, self._args.min_length_spacer,
            self._args.Max_length_spacer, self._paths.crispr_output_folder,
            self._args.ignore_hypothetical_protein)
        cris = Crispr(args_cris)
        cris.run_crispr(args_cris)

    def merge(self):
        """Merge all features"""
        print("Merging all features to one gff file")
        merge_folder = os.path.join(self._paths.output_folder,
                                    "merge_all_features")
        self.helper.check_make_folder(merge_folder)
        other_features = self._args.other_features_files
        self.check_file([self._args.transcript_file] + other_features,
                        ["--transcript_file", "--other_features_files"],
                        False)
        self.check_parameter([self._args.output_prefix], ["--output_prefix"])
        run_merge(merge_folder, self._args.transcript_file,
                  self._args.other_features_files,
                  self._args.terminator_tolerance, self._args.tss_tolerance,
                  os.path.join(merge_folder, self._args.output_prefix))

    def screen(self):
        """generate screenshot"""
        print("Running screenshot generation")
        self.check_file([self._args.main_gff, self._args.fasta_file],
                        ["--main_gff", "--fasta_file"], True)
        if self._args.side_gffs is not None:
            for gff in (self._args.side_gffs):
                gff = gff.strip()
                if not os.path.isfile(gff):
                    print("Error: The --side_gffs do not exist!")
                    sys.exit()
        if self._args.output_folder is None:
            print("Error: Please assign --output_folder!")
            sys.exit()
        if (self._args.present != "expand") and (
                self._args.present != "collapse") and (
                self._args.present != "squish"):
            print("Error: Please assign \"expand\" or "
                  "\"collapse\" or \"squish\" to --present!")
            sys.exit()
        args_sc = self.args_container.container_screen(
            self._args.main_gff, self._args.side_gffs,
            self._args.fasta_file, self._args.height,
            self._args.tex_notex_libs, self._args.frag_libs,
            self._args.present, self._args.output_folder)
        screen = Screen(args_sc)
        screen.screenshot(args_sc)



from annogesiclib.gff3 import Gff3Parser


def get_type(inter, gffs):
    '''get the types of sORF'''
    utr5 = False
    utr3 = False
    anti = False
    for gff in gffs:
        if (gff.seq_id == inter["strain"]) and \
           (gff.strand == inter["strand"]):
            if gff.strand == "+":
                if inter["end"] + 1 == gff.start:
                    utr5 = True
                if inter["start"] - 1 == gff.end:
                    utr3 = True
            else:
                if inter["end"] + 1 == gff.start:
                    utr3 = True
                if inter["start"] - 1 == gff.end:
                    utr5 = True
        elif (gff.seq_id == inter["strain"]) and \
             (gff.strand != inter["strand"]):
            if ((inter["start"] <= gff.start) and (
                 inter["end"] >= gff.end)) or (
                (inter["start"] >= gff.start) and (
                 inter["end"] <= gff.end)) or (
                (inter["start"] <= gff.start) and (
                 inter["end"] <= gff.end) and (
                 inter["end"] >= gff.start)) or (
                (inter["start"] >= gff.start) and (
                 inter["start"] <= gff.end) and (
                 inter["end"] >= gff.end)):
                anti = True
    if utr3 and utr5:
        inter["source"] = "interCDS"
    elif utr3:
        inter["source"] = "3utr"
    elif utr5:
        inter["source"] = "5utr"
    elif anti:
        inter["source"] = "antisense"
    else:
        inter["source"] = "intergenic"


def read_gff(gff_file, tran_file, hypo):
    trans = []
    gffs = []
    gh = open(gff_file)
    for entry in Gff3Parser().entries(gh):
        if (entry.feature == "CDS") or \
           (entry.feature == "rRNA") or \
           (entry.feature == "tRNA") or \
           (entry.feature == "sRNA"):
            if ("product" in entry.attributes.keys()) and (hypo):
                if "hypothetical protein" not in entry.attributes["product"]:
                    gffs.append(entry)
            else:
                gffs.append(entry)
    th = open(tran_file)
    for entry in Gff3Parser().entries(th):
        trans.append(entry)
    gffs = sorted(gffs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    trans = sorted(trans, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    gh.close()
    th.close()
    return gffs, trans


def compare_tran_cds(trans, gffs):
    '''compare transcript and CDS to find the intergenic region'''
    inters = []
    for tran in trans:
        poss = [{"start": tran.start, "end": tran.end}]
        for pos in poss:
            exclude = False
            for gff in gffs:
                if (tran.seq_id == gff.seq_id) and \
                   (tran.strand == gff.strand):
                    if (gff.start <= pos["start"]) and \
                       (gff.end >= pos["start"]) and \
                       (gff.end < pos["end"]):
                        pos["start"] = gff.end + 1
                    elif (gff.start > pos["start"]) and \
                         (gff.start <= pos["end"]) and \
                         (gff.end >= pos["end"]):
                        pos["end"] = gff.start - 1
                    elif (gff.start <= pos["start"]) and \
                         (gff.end >= pos["end"]):
                        exclude = True
                        break
                    elif (gff.start > pos["start"]) and \
                         (gff.end < pos["end"]):
                        poss.append({"start": gff.end + 1, "end": pos["end"]})
                        pos["end"] = gff.start - 1
            if not exclude:
                inters.append({"strain": tran.seq_id, "strand": tran.strand,
                               "start": pos["start"], "end": pos["end"]})
    return inters


def get_intergenic(gff_file, tran_file, out_file, utr_detect, hypo):
    gffs, trans = read_gff(gff_file, tran_file, hypo)
    inters = compare_tran_cds(trans, gffs)
    num = 0
    out = open(out_file, "w")
    for inter in inters:
        get_type(inter, gffs)
        name = '%0*d' % (5, num)
        if (inter["source"] != "intergenic") and (
                inter["source"] != "antisense"):
            source = "UTR_derived"
            if utr_detect:
                attribute_string = ";".join(
                       ["=".join(items) for items in (
                           ["ID", inter["strain"] + "_sorf" + str(num)],
                           ["Name", "sORF_" + name],
                           ["UTR_type", inter["source"]])])
        else:
            if inter["source"] == "intergenic":
                source = "intergenic"
            elif inter["source"] == "antisense":
                source = "antisense"
            attribute_string = ";".join(
                   ["=".join(items) for items in (
                       ["ID", inter["strain"] + "_sorf" + str(num)],
                       ["Name", "sORF_" + name])])
        if ((source == "UTR_derived") and (utr_detect)) or (
                source == "intergenic") or (source == "antisense"):
            out.write("\t".join([str(field) for field in [
                        inter["strain"], source, "sORF", str(inter["start"]),
                        str(inter["end"]), ".", inter["strand"], ".",
                        attribute_string]]) + "\n")
        num += 1
    out.close()



import csv


class TSSPredatorReader(object):

    def entries(self, input_fh):
        for row in csv.reader(input_fh, delimiter="\t"):
            if row[0].startswith("SuperPos"):
                continue
            yield TSSPredatorEntry(row)


class TSSPredatorEntry(object):

    def __init__(self, row):
        assert(len(row) == 28)
        self.super_pos = int(row[0])
        self.super_strand = row[1]
        self.map_count = int(row[2])
        self.det_count = int(row[3])
        self.genome = row[4]
        self.is_detected = True if row[5] == "1" else False
        self.is_enriched = True if row[6] == "1" else False
        self.step_heigth = row[7]
        self.step_factor = row[8]
        self.enrichment_factor = row[9]
        self.class_count = int(row[10])
        self.pos = int(row[11])
        self.strand = row[12]
        self.locus_tag = row[13]
        self.srna_asrna = row[14]
        self.product = row[15]
        self.utr_length = row[16]
        self.gene_length = row[17]
        self.is_primary = True if row[18] == "1" else False
        self.is_secondary = True if row[19] == "1" else False
        self.is_internal = True if row[20] == "1" else False
        self.is_antisense = True if row[21] == "1" else False
        self.is_automated = True if row[22] == "1" else False
        self.is_manual = True if row[23] == "1" else False
        self.is_putative_srna = True if row[24] == "1" else False
        self.is_putative_asrna = True if row[25] == "1" else False
        self.comment = row[26]
        self.seq = row[27]
        self.is_orphan = False
        if (self.is_primary is False and self.is_secondary is False and
                self.is_internal is False and self.is_antisense is False):
            self.is_orphan = True

    def __str__(self):
        return "%s %s %s" % (self.super_pos, self.super_strand, self.genome)
